package org.jfox.ioc.management.http;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.apache.velocity.context.Context;
import org.jfox.ioc.ComponentMeta;
import org.jfox.ioc.Registry;

/**
 * @author <a href="mailto:yy.young@gmail.com">Young Yang</a>
 */

public class IndexServlet extends SuperServlet {

    public String getTemplate() {
        return "index.vm";
    }

    public void buildContext(HttpServletRequest request, Context ctx) throws Exception {

        HttpSession session = request.getSession(true);
        session.setAttribute("time",System.currentTimeMillis() + "");

        Registry registry = getRegistry();

        List<ComponentMeta> metas = registry.getAllComponentMetas();
        Map<String,List> modules = new TreeMap<String,List>();
        for(ComponentMeta meta : metas){
            String module = meta.getModule();
            String moduleKey = "<b>" + meta.getModuleDesc() + "</b> - "+ module;
            if(modules.containsKey(moduleKey)){
                List list = (List)modules.get(moduleKey);
                list.add(meta);
                Collections.sort(list);
            }
            else {
                List list = new ArrayList();
                list.add(meta);
                modules.put(moduleKey,list);
            }
        }

        ctx.put("title","Index View");
        ctx.put("modules",modules);
        ctx.put("moduleCounts",modules.size());
        ctx.put("typeCounts",registry.countComponentTypes());
        ctx.put("componentCounts",metas.size());
    }

    public static void main(String[] args) {

    }
}

