package org.jfox.ioc.management.http;

import java.lang.reflect.Method;
import java.util.List;
import java.util.Map;
import javax.servlet.http.HttpServletRequest;

import org.apache.velocity.context.Context;
import org.jfox.ioc.ComponentMeta;
import org.jfox.ioc.ComponentName;
import org.jfox.ioc.management.ManagementInfo;
import org.jfox.ioc.management.ManagementOperationInfo;
import org.jfox.ioc.util.Classes;

/**
 * @author <a href="mailto:yy.young@gmail.com">Young Yang</a>
 */

public class InvokeServlet extends SuperServlet {


    public String getTemplate() {
        return "resultview.vm";
    }

    public void buildContext(HttpServletRequest request, Context ctx) throws Exception {
        String operHashCode = request.getParameter("operHashCode");
        String _componentName = request.getParameter("componentname");
        ComponentName componentName = ComponentName.parseString(_componentName,getRegistry().getClassLoaderRepository());

        ComponentMeta meta = getRegistry().getComponentMeta(componentName);
        ManagementInfo minfo = meta.getManagementInfo();
        ManagementOperationInfo operInfo = minfo.getManagementOperationInfo(operHashCode);
        Method method = operInfo.getMethod();
        Class[] paramTypes = method.getParameterTypes();
        String[] _values = new String[paramTypes.length];
        for(int i = 0; i < paramTypes.length; i++) {
            String _value = request.getParameter(operInfo.getName() + "_" + i);
            if(_value == null) {
                _value = "";
            }
            _values[i] = _value.trim();
        }

        Object result = null;

        Object[] values = createObjects(_values, paramTypes);
        result = method.invoke(getRegistry().getComponentInstance(componentName), values);

        ctx.put("title","Result View");
        ctx.put("componentName",meta.getComponentName());
        ctx.put("operInfo",operInfo);
        ctx.put("result",result == null ? "" : result);

        ctx.put("isList", false);
        ctx.put("isMap", false);

        if(result != null) {
            if(result.getClass().isArray() || (result instanceof List) ) {
                ctx.put("isList", true);
            }
            else if(result instanceof Map) {
                ctx.put("isMap", true);
            }
        }
        else {
        }

    }

    private Object[] createObjects(String[] values, Class[] signatures) throws Exception {
        Object[] objects = new Object[values.length];
        for(int i = 0; i < objects.length; i++) {
            Object obj = Classes.newObject(signatures[i], values[i]);
            objects[i] = obj;
        }
        return objects;
    }

    public static void main(String[] args) {

    }
}

