/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */

package org.jfox.ioc;

import java.util.ArrayList;
import java.util.List;

import org.jfox.ioc.depend.Dependency;
import org.jfox.ioc.depend.DependencyPack;
import org.jfox.ioc.depend.RefDependency;
import org.jfox.ioc.exception.ComponentException;
import org.jfox.ioc.ext.ManagableComponent;
import org.jfox.ioc.ext.SingletonComponent;
import org.jfox.ioc.factory.ComponentFactory;
import org.jfox.ioc.factory.ConstrComponentFactory;
import org.jfox.ioc.factory.InstanceComponentFactory;
import org.jfox.ioc.factory.PropertyComponentFactory;
import org.jfox.ioc.logger.Logger;
import org.jfox.ioc.management.ManagementInfo;

/**
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public class ComponentMeta implements Comparable {

    public final static boolean DEFAULT_SINGLETON = true;

    public final static boolean DEFAULT_TYPE_SINGLETON = false;

    public final static int DEFAULT_PRIORITY = 0;

    private static Logger logger = Logger.getLogger(ComponentMeta.class
            .getName());

    /**
     * 
     */
    protected String description = "";

    /**
     * Ƿ trueʱRegistryֻɸһʵ
     * falseʱRegistryΪÿgetComponentInstanceһʵ ĬΪ true <br>
     * <br>
     * ע⣺ԶʽעᵽRegistryУôضsingleton
     */
    protected boolean singleton = DEFAULT_SINGLETON;

    /**
     * ͵ֻRegistryעһΣࡣ  B extents AAע֮󣬾Ͳٴעˣ B
     * Ȼעᡣ
     */
    protected boolean typeSingleton = DEFAULT_TYPE_SINGLETON;

    /**
     * ȼģ鷽ʽʱע˳ ȼߵעᵽRegistryУֵСȼ
     */
    protected int priority = DEFAULT_PRIORITY;

    /**
     * ɸʵFactory
     */
    protected ComponentFactory compFactory;

    /**
     * ʵsingleton򻺴ʵǷsingleton򻺴һʵ
     */
    protected Component component = null;

    /**
     * õбʵ֮󣬲ֵ
     */
    protected List refComponentNames = new ArrayList();

    private String module = "";
    /**
     * ڵģmodule
     */
    private String moduleDesc = "Default Module";

    /**
     * ڵģڵĿ¼
     */
    private String moduleDir = null;

    protected ComponentMeta(ComponentFactory compFactory) {
        this.compFactory = compFactory;
    }

    /**
     * /** Ƿ trueʱRegistryֻɸһʵ
     * falseʱRegistryΪÿgetComponentInstanceһʵ ĬΪ true <br>
     * <br>
     * ע⣺ԶʽעᵽRegistryУôضsingletonģ ʵ{@link SingletonComponent}ôҲضsingleton
     */
    public boolean isSingleton() {
        return SingletonComponent.class
                .isAssignableFrom(getComponentImplementation())
                || singleton;
    }

    public void setSingleton(boolean singleton) {
        // ʹ InstanceComponentFactory  singletonֻǵ
        if (!(compFactory instanceof InstanceComponentFactory)) {
            this.singleton = singleton;
        }
    }

    public boolean isTypeSingleton() {
        return typeSingleton;
    }

    public void setTypeSingleton(boolean typeSingleton) {
        this.typeSingleton = typeSingleton;
    }

    public int getPriority() {
        return priority;
    }

    public void setPriority(int priority) {
        this.priority = priority;
    }

    /**
     * get component instace by meta attention: will not init & start component
     * by Registry!!!
     * 
     * @param paramPacks
     *            ʵҪеsettersetter֮󣬲Ž initstart 
     * @return
     * @throws ComponentException
     */
    public synchronized Component getComponentInstance(
            DependencyPack[] paramPacks) throws ComponentException {
        logger.debug("get component instance "
                + getComponentFactory().getComponentName());

        if (isSingleton()) {
            if (component == null) {
                logger.debug("make new component instance, name: "
                        + getComponentFactory().getComponentName()
                        + ", component factory: " + getComponentFactory());
                ComponentFactory factory = getComponentFactory();
                if (paramPacks != null && paramPacks.length != 0) {
                    if (factory instanceof ConstrComponentFactory) {
                        factory = new PropertyComponentFactory(
                                (ConstrComponentFactory) factory, paramPacks);
                    }
                    else if (factory instanceof PropertyComponentFactory) {
                        ((PropertyComponentFactory) factory)
                                .addParameterPack(paramPacks);
                    }
                }

                ComponentContext context = initComponentContext();
                if(factory instanceof PropertyComponentFactory){
                    // properryComponentFactory will set ComponentContext before setters
                    component = ((PropertyComponentFactory)factory).makeComponent(context);
                }
                else {
                    // set component ctx
                    component = factory.makeComponent();
                    component.setComponentContext(context);
                }
                logger.debug("make new instance: " + component);
                buildRefComponentNames();

                // try to int & start only when created
                if (getRegistry().isStarted()) {
                    getRegistry().initComponent(this, component);
                    getRegistry().startComponent(this, component);
                }
            }
            else {
                if (paramPacks != null && paramPacks.length != 0) {
                    ComponentFactory factory = getComponentFactory();
                    factory = new PropertyComponentFactory(
                            new InstanceComponentFactory(component), paramPacks);
                    component = factory.makeComponent();
                }
            }
        }
        else {
            logger.debug("make new component instance with component name "
                    + getComponentFactory().getComponentName()
                    + ",component factory " + getComponentFactory());
            ComponentFactory factory = getComponentFactory();
            if (paramPacks != null && paramPacks.length != 0) {
                if (factory instanceof ConstrComponentFactory) {
                    factory = new PropertyComponentFactory(
                            (ConstrComponentFactory) factory, paramPacks);
                }
                else if (factory instanceof PropertyComponentFactory) {
                    ((PropertyComponentFactory) factory)
                            .addParameterPack(paramPacks);
                }
            }

            ComponentContext context = initComponentContext();
            Component comp = null;
            if(factory instanceof PropertyComponentFactory) {
                // properryComponentFactory will set ComponentContext before setters
                comp = ((PropertyComponentFactory) factory).makeComponent(context);
            }
            else {
                // set component ctx
                comp = factory.makeComponent();
                comp.setComponentContext(context);
            }

            // only build at the first time
            if(component == null) {
                buildRefComponentNames();
            }
            component = comp;

            // try to int & start only when created
            if (getRegistry().isStarted()) {
                getRegistry().initComponent(this, component);
                getRegistry().startComponent(this, component);
            }

        }
        return component;
    }

    // find the depency component ref
    private void buildRefComponentNames() {
        // properties ref
        if (compFactory instanceof PropertyComponentFactory) {
            DependencyPack[] packs = ((PropertyComponentFactory) compFactory)
                    .getParameterPacks();
            for (int i = 0; packs != null && i < packs.length; i++) {
                Dependency param = packs[i].getParameter();
                if (param instanceof RefDependency) {
                    refComponentNames.add(((RefDependency) param)
                            .getRefComponentName());
                }
            }
        }

        // constructor ref
        Dependency[] params = getComponentFactory().getParameters();
        for (int i = 0; params != null && i < params.length; i++) {
            Dependency param = params[i];
            if (param instanceof RefDependency) {
                refComponentNames.add(((RefDependency) param)
                        .getRefComponentName());
            }
        }

    }

    public ComponentName getComponentName() {
        return getComponentFactory().getComponentName();
    }

    public Class getComponentImplementation() {
        return getComponentFactory().getImplementation();
    }

    public ComponentFactory getComponentFactory() {
        return compFactory;
    }

    public ComponentName[] getRefComponentNames() {
        return (ComponentName[]) refComponentNames
                .toArray(new ComponentName[refComponentNames.size()]);
    }

    /**
     * return the Registry return null if this Component is not registry yet
     * 
     * @return
     */
    public Registry getRegistry() {
        return getComponentFactory().getRegistry();
    }

    void setRegistry(Registry registry) {
        getComponentFactory().setRegistry(registry);
    }

    public ManagementInfo getManagementInfo() {
        if (!isSingleton()) {
            return null;
        }
        else if (component == null) {
            return null;
        }
        else {
            if (component instanceof ManagableComponent) {
                return ManagementInfo.getManagementInfoByAnnotation(component
                        .getClass());
            }
            else {
                return null;
            }
        }
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    /**
     * @return Returns the module.
     */
    public String getModule() {
        return module;
    }
    

    /**
     * @param module The module to set.
     */
    public void setModule(String module) {
        this.module = module;
    }
    

    public String getModuleDesc() {
        return moduleDesc;
    }

    public void setModuleDescription(String module) {
        this.moduleDesc = module;
    }

    public String getModuleDir() {
        return moduleDir;
    }

    public void setModuleDir(String moduleDir) {
        this.moduleDir = moduleDir;
    }

    private ComponentContext initComponentContext() {
        ComponentContext ctx = new ComponentContext();
        if (moduleDir == null) {
            // if register by manual, may not set module, so use the classpath
            // as
            moduleDir = getComponentImplementation().getProtectionDomain()
                    .getCodeSource().getLocation().getPath();
        }
        ctx.setRegistry(getRegistry());
        ctx.setModule(module);
        ctx.setModuleDesc(moduleDesc);
        ctx.setModuleDir(moduleDir);
        ctx.setSingleton(singleton);
        ctx.setTypeSingleton(typeSingleton);
        return ctx;
    }

    /**
     * priority ֵ ComponentMeta  priority Сȼ Registry ʱ򣬻ע
     * 
     * @param o
     * @return
     */
    public int compareTo(Object o) {
        if (!(o instanceof ComponentMeta))
            return 0;
        ComponentMeta cm = (ComponentMeta) o;
        if (priority > cm.priority) {
            return 1;
        }
        else if (priority < cm.priority) {
            return -1;
        }
        else {
            return getComponentName().toString().compareTo(
                    cm.getComponentName().toString());
        }
    }

    public String toString() {
        return "ComponentMeta{" + "componentType="
                + getComponentImplementation().getName() + ", compFactory="
                + compFactory.toString() + ", singleton=" + singleton
                + ", typeSingleton=" + typeSingleton + "}";
    }

}