/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */

package org.jfox.ioc;

import org.jfox.ioc.depend.Dependency;
import org.jfox.ioc.depend.DependencyPack;
import org.jfox.ioc.exception.ComponentException;
import org.jfox.ioc.ext.InterceptableComponent;
import org.jfox.ioc.factory.ComponentFactory;
import org.jfox.ioc.factory.ConstrComponentFactory;
import org.jfox.ioc.factory.DynProxyComponentFactory;
import org.jfox.ioc.factory.InstanceComponentFactory;
import org.jfox.ioc.factory.PropertyComponentFactory;

/**
 *  ComponentMeta ĹԸûṩĲԶComponentMetaҪComponentName  ComponentFactory 
 *
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public class ComponentMetaFactory {

    /**
     * ʵComponentMeta
     * ַʽɵComponentMetasingletonֻtrue
     * @param component ʵ
     */
    public static ComponentMeta getComponentMeta(Component component) {
        return new ComponentMeta(new InstanceComponentFactory(component));
    }

    /**
     * ComponentMetaʹĬֵ
     * ʹûвĬϹ
     * @param implementation
     * @return
     * @throws ComponentException
     */
    public static ComponentMeta getComponentMeta(Class implementation) throws ComponentException {
        return getComponentMeta(implementation,implementation);
    }

    public static ComponentMeta getComponentMeta(Class superClass,Class implementation) throws ComponentException {
        return getComponentMeta(superClass,implementation, Dependency.EMPTY_PARAMETERS);
    }

    public static ComponentMeta getComponentMeta(Class implementation, Dependency[] constructorTypes) throws ComponentException {
        return getComponentMeta(implementation,implementation,constructorTypes,DependencyPack.EMPTY_PARAMETER_PACKS,true,false);
    }

    public static ComponentMeta getComponentMeta(Class superClass,Class implementation, Dependency[] constructorTypes) throws ComponentException {
        return getComponentMeta(superClass,implementation, constructorTypes, DependencyPack.EMPTY_PARAMETER_PACKS, true, false);
    }

    public static ComponentMeta getComponentMeta(Class superClass,
                                                 Class implementation,
                                                 Dependency[] constructTypes,
                                                 DependencyPack[] setterParams,
                                                 boolean singleton,
                                                 boolean typeSingleton) throws ComponentException {
        ComponentName name = ComponentName.newInstance(superClass);
        return getComponentMeta(name,implementation,constructTypes,setterParams,singleton,typeSingleton);
    }

    /**
     * ʵComponentMeta typeSingleton
     * ַʽɵComponentMetasingletonֻtrue
     * @param component ʵ
     * @param typeSingleton ൥ֻעһ
     */
    public static ComponentMeta getComponentMeta(Component component, boolean typeSingleton) {
        ComponentMeta meta = getComponentMeta(component);
        meta.setTypeSingleton(typeSingleton);
        return meta;
    }

    /**
     * ComponentNameComponentMetaʹĬֵ.
     * ʹûвĬϹ
     * @param name עƣѾ
     * @throws ComponentException
     */
    public static ComponentMeta getComponentMeta(ComponentName name)
            throws ComponentException {
        return getComponentMeta(name,name.getType());
    }

    public static ComponentMeta getComponentMeta(ComponentName name, Class implementation)
            throws ComponentException {
        return getComponentMeta(name,implementation,Dependency.EMPTY_PARAMETERS);
    }

    public static ComponentMeta getComponentMeta(ComponentName name,
                                                 Dependency[] constructorTypes)
            throws ComponentException {
        return getComponentMeta(name,name.getType(),constructorTypes);
    }

    public static ComponentMeta getComponentMeta(ComponentName name,
                                                 Class implementation,
                                                 Dependency[] constructorTypes)
            throws ComponentException {
        return getComponentMeta(name,implementation,constructorTypes, DependencyPack.EMPTY_PARAMETER_PACKS);
    }
    /**
     * ơsetterComponentMetaʹĬֵ
     *
     * @param name עƣѾ
     * @param constructorTypes 
     * @throws ComponentException
     */
    public static ComponentMeta getComponentMeta(ComponentName name,
                                                 Class implementation,
                                                 Dependency[] constructorTypes,
                                                 DependencyPack[] setterParams
                                                    ) throws ComponentException {
        return getComponentMeta(name,implementation,constructorTypes,setterParams,true,false);
    }

    public static ComponentMeta getComponentMeta(ComponentName name,
                                                 Dependency[] constructCompParamPack,
                                                 DependencyPack[] setterParams,
                                                 boolean singleton,
                                                 boolean typeSingleton
                                                 )

            throws ComponentException {
        return getComponentMeta(name,name.getType(),constructCompParamPack,setterParams,singleton,typeSingleton);
    }

    public static ComponentMeta getComponentMeta(ComponentName name,
                                                 Class implementation,
                                                 Dependency[] constrctCompParamPack,
                                                 DependencyPack[] setterParams,
                                                 boolean singleton,
                                                 boolean typeSingleton)

            throws ComponentException {
        ComponentFactory componentFactory = null;
        Class superClass = name.getType();
        if(!superClass.isAssignableFrom(implementation)){
            throw new ComponentException("component name " + name + " 's super class " + superClass.getName() + " is not the superClass the implementation class " + implementation.getName());
        }

        if(InterceptableComponent.class.isAssignableFrom(implementation)) {
            componentFactory = new DynProxyComponentFactory(name, implementation, constrctCompParamPack);
        }
        else {
            componentFactory = new ConstrComponentFactory(name, implementation, constrctCompParamPack);
        }
        if(setterParams != null && setterParams.length != 0) {
            componentFactory = new PropertyComponentFactory((ConstrComponentFactory) componentFactory, setterParams);
        }

        ComponentMeta meta = new ComponentMeta(componentFactory);
        meta.setTypeSingleton(typeSingleton);
        meta.setSingleton(singleton);
        return meta;
    }

    public static void main(String[] args) {

    }
}

