/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */
package org.jfox.ioc;

import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.jfox.ioc.classloader.ClassLoaderRepository;
import org.jfox.ioc.depend.Dependency;
import org.jfox.ioc.depend.DependencyPack;
import org.jfox.ioc.exception.ComponentException;
import org.jfox.ioc.ext.ActiveComponent;
import org.jfox.ioc.ext.InitializableComponent;
import org.jfox.ioc.ext.ServicableComponent;
import org.jfox.ioc.logger.Logger;

/**
 * Registryʵ࣬½עȹܡ
 * <br>
 * һ:
 * <pre>
 *      Registry registry = Registry.getInstance();
 *      registry.registerComponent(...);
 *      registry.registerComponent(...);
 *      ...
 *      //  IoC ContainerҪʼҪгʼ
 *      registry.start();
 * </pre>
 *
 *
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public class Registry {

    /**
     * IoC container Ƿ
     */
    private boolean started = false;

    /**
     * ־¼
     */
    private Logger logger = Logger.getLogger(Registry.class.getName());

    /**
     * ClassLoader 
     */
    private ClassLoaderRepository loaderRepo = new ClassLoaderRepository(new URL[0], Thread.currentThread().getContextClassLoader());

    /**
     * ʵ
     */
    private static final Registry me = new Registry();
    private final Repository repo = new Repository();

    /**
     * õRegistryĵ
     */
    public static Registry getInstance() {
        return me;
    }

    private Registry() {
        // õǰ̵߳ ClassLoader
        Thread.currentThread().setContextClassLoader(loaderRepo);
    }

    /**
     * ʵעһԶעǵ
     * ϵͳComponentComponentNameComponentNameΪhashCode
     * Type singleton ĬΪ trueֻעһΡ
     *
     * @see #registerComponent(Component, boolean)
     * @param component Ҫעʵ
     * @throws ComponentException
     */
    public synchronized ComponentMeta registerComponent(Component component) throws ComponentException {
        return registerComponent(component,false);
    }

    /**
     * ʵעһԶעǵ
     * ϵͳComponentComponentNameComponentNameΪhashCode
     *
     * @param component ʵ
     * @param typeSingleton Ƿֻעһ
     * @return
     * @throws ComponentException
     */
    public synchronized ComponentMeta registerComponent(Component component, boolean typeSingleton) throws ComponentException {
        ComponentMeta meta = ComponentMetaFactory.getComponentMeta(component, typeSingleton);
        return registerComponent(meta);
    }

    /**
     * עһ
     * ϵͳimplementationComponentNameComponentNameΪimplementationhashCode
     * вʹĬã
     * <br>
     *  1.ʹòĬϹ
     *  2.û setter
     *  3.singleton Ϊ true
     *  4.type singleton Ϊ true
     * <br><br>
     * ע⣺ʵ {@link Component} ӿ
     *
     * @param implementation
     * @throws ComponentException
     */
    public synchronized ComponentMeta registerComponent(Class implementation) throws ComponentException {
        return registerComponent(ComponentName.newInstance(implementation), implementation);
    }

    public synchronized ComponentMeta registerComponent(Class implementation, Dependency[] constructParams) throws ComponentException {
        return registerComponent(implementation, constructParams, DependencyPack.EMPTY_PARAMETER_PACKS, true, false);
    }

    /**
     * עһ
     * ϵͳimplementationComponentNameComponentNameΪimplementationhashCode
     * вʹĬã
     * <br>
     * 1.ʹòĬϹ
     * 2.û setter
     * 3.singleton Ϊ true
     * 4.type singleton Ϊ true
     * <br><br>
     * ע⣺ʵ {@link Component} ӿ
     *
     * @param implementation
     * @throws ComponentException
     */
    public synchronized ComponentMeta registerComponent(Class implementation, Dependency[] constructorParams, DependencyPack[] setterParams, boolean singleton, boolean typeSingleton) throws ComponentException {
        ComponentMeta meta = ComponentMetaFactory.getComponentMeta(ComponentName.newInstance(implementation), implementation, constructorParams, setterParams, singleton, typeSingleton);
        registerComponent(meta);
        return meta;

    }
    
    public synchronized ComponentMeta registerComponent(ComponentName componentName,Class implementation) throws ComponentException {
        return registerComponent(componentName, implementation,Dependency.EMPTY_PARAMETERS);
    }

    public synchronized ComponentMeta registerComponent(ComponentName componentName,Class implementation, Dependency[] constructParams) throws ComponentException {
        return registerComponent(componentName, implementation,constructParams, DependencyPack.EMPTY_PARAMETER_PACKS, true, false);
    }

/*    public synchronized ComponentMeta registerComponent(ComponentName componentName, Parameter[] constructParams, ParameterPack[] setterParams, boolean singleton, boolean typeSingleton) throws ComponentException {
        ComponentMeta componentMeta = ComponentMetaFactory.getComponentMeta(componentName,constructParams,setterParams,singleton,typeSingleton);
        return registerComponent(componentMeta);
    }*/

    public synchronized ComponentMeta registerComponent(ComponentName componentName,Class implementation, Dependency[] constructParams, DependencyPack[] setterParams, boolean singleton, boolean typeSingleton) throws ComponentException {
        ComponentMeta componentMeta = ComponentMetaFactory.getComponentMeta(componentName,implementation,constructParams, setterParams, singleton, typeSingleton);
        return registerComponent(componentMeta);
    }

    /**
     *  ComponentMeta עһComponentMeta ûֶӦʹ {@link ComponentMetaFactory} ComponentMeta
     */
    public synchronized ComponentMeta registerComponent(ComponentMeta componentMeta) throws ComponentException {
        doRegister(componentMeta);
        doActive(componentMeta);
        return componentMeta;
    }

    /**
     * עComponentҪComponentDepoyerעModuleComponent
     * ΪActiveComponentµļ˳
     * @param componentMetas
     * @return success component metas
     */
    public synchronized ComponentMeta[] registerComponentBatch(ComponentMeta[] componentMetas) {
        List metas = new ArrayList(componentMetas.length);
        if(componentMetas != null && componentMetas.length > 0) {
            for(int i=0; i<componentMetas.length; i++){
                try {
                    ComponentMeta meta = componentMetas[i];
                    doRegister(meta);
                    metas.add(meta);
                }
                catch(ComponentException e){
                    logger.warn(e.getMessage(),e);
                }
            }
        }
        for(int i=0;i<metas.size(); i++){
            ComponentMeta meta = (ComponentMeta)metas.get(i);
            try {
                doActive(meta);
            }
            catch(ComponentException e){
                logger.warn(e.getMessage(),e);
            }
        }
        return (ComponentMeta[])metas.toArray(new ComponentMeta[metas.size()]);
    }


    /**
     *  ComponentName õ ComponentMeta
     * @param name ע
     */
    public synchronized ComponentMeta getComponentMeta(ComponentName name) throws ComponentException {
        ComponentMeta meta = repo.getComponentMeta(name);
        if(meta == null) {
            throw new ComponentException("component " + name + " not found.");
        }
        return meta;
    }

    /**
     * Componentע
     * ComponentNameΪhashCode
     *
     * @param implementaion
     * @return
     * @throws ComponentException
     */
    public Component getComponentInstance(Class implementaion) throws ComponentException {
        ComponentName name = ComponentName.newInstance(implementaion);
        return getComponentInstance(name);
    }

    /**
     * עƵõʵ
     *
     * @param name ע
     * @return
     * @throws ComponentException
     */
    public synchronized Component getComponentInstance(ComponentName name) throws ComponentException {
        return getComponentInstance(name, DependencyPack.EMPTY_PARAMETER_PACKS);
    }

    /**
     * Component Instance ֮󣬻Ͻ ParameterPack ע
     * ҪЩʱֵܸ
     *
     * @param name
     * @param paramPacks
     * @return
     * @throws ComponentException
     */
    public synchronized Component getComponentInstance(ComponentName name, DependencyPack[] paramPacks) throws ComponentException {
        ComponentMeta meta = null;
        if(!isRegistered(name)) {
            throw new ComponentException("component " + name + " not registered.");
        }
        meta = getComponentMeta(name);

        Component comp = meta.getComponentInstance(paramPacks);
        return comp;
    }


    /**
     * עעĳ
     * ǵģ Initializable / Serviceable stop / destory 
     * עɹ ComponentMetaڣ null
     *
     * @see InitializableComponent
     * @see ServicableComponent
     * @param name ע
     * @throws ComponentException
     */
    public synchronized ComponentMeta unregisterComponent(ComponentName name) throws ComponentException {
        logger.info("unregister component " + name);
        ComponentMeta meta = repo.unregisterComponent(name);
        // only stop & destroy singlton component, otherwise, must done by yourself
        if(meta != null && meta.isSingleton()) {
            Component comp = meta.getComponentInstance(DependencyPack.EMPTY_PARAMETER_PACKS);
            stopComponent(meta, comp);
            destroyComponent(meta, comp);
        }
        return meta;
    }

    /**
     * صǰעĿ
     */
    public int countComponents() {
        return repo.countComponents();
    }

    /**
     * صǰע˶͵
     */
    public int countComponentTypes() {
        return repo.countComponentTypes();
    }

    /**
     * ͵ǷעᵽRegistryС
     * @param implementation
     */
    public boolean isRegisteredImplemetation(Class implementation) {
        return repo.isRegisteredImplemetation(implementation);
    }

    /**
     * ComponentName ǷѾע
     * @param name ע
     */
    public boolean isRegistered(ComponentName name) {
        return repo.isRegistered(name);
    }

    /**
     * õע
     */
    public synchronized List<ComponentName> getAllComponentNames() {
        return repo.getAllComponentNames();
    }

    public synchronized List<ComponentMeta> getAllComponentMetas() {
        return repo.getAllComponentMetas();
    }


    /**
     * õԸעԸע
     *
     * @param type 
     */
    public List<ComponentName> getComponentNamesByImplemetation(Class type) {
        return repo.getComponentNamesByImplemetation(type);
    }

    /**
     * õԸעƣԸע
     * ڸݲԶѰʱҪõ
     *
     * @param implementaion 
     */
    public synchronized ComponentName getFirstComponentNameByImplemetation(Class implementaion) {
        return repo.getFirstComponentNameByType(implementaion);
    }

    /**
     * õԸעƣԸע
     * @param implementaion 
     */
    public ComponentName getLastComponentNameByImplemetation(Class implementaion) {
        return repo.getLastComponentNameByType(implementaion);
    }

    /**
     *  Registryעᵽе Initializable  Serviceable ĵгʼ
     */
    public synchronized void start() {
        if(started) return;
        logger.info("start Registry.");
        List<ComponentMeta> compMetas = new ArrayList<ComponentMeta>(repo.getAllComponentMetas());
        Collections.sort(compMetas);

        for(ComponentMeta meta : compMetas){
            try {
                Component comp = meta.getComponentInstance(DependencyPack.EMPTY_PARAMETER_PACKS);
                try {
                    initComponent(meta, comp);
                }
                catch(ComponentException e) {
                    logger.warn("init component error", e);
                }
                try {
                    startComponent(meta, comp);
                }
                catch(ComponentException e) {
                    logger.warn("start component error", e);
                }
            }
            catch(Exception e) {
                logger.warn("get component instance " + meta.getComponentName() + "failed.", e);
            }
        }
        started = true;
    }

    /**
     * ֹͣ Registryעᵽе Initializable  Serviceable ĵٺֹͣ
     */
    public synchronized void stop() {
        if(started == false) return;
        logger.info("stop Registry.");
        List<ComponentMeta> compMetas = new ArrayList<ComponentMeta>(repo.getAllComponentMetas());
        Collections.sort(compMetas);
        Collections.reverse(compMetas);

        for(ComponentMeta meta : compMetas){
            try {
                //ֻ singleton  Component  stop & destroyȴ stop & destroy 
                if(meta.isSingleton()) {
                    Component comp = meta.getComponentInstance(DependencyPack.EMPTY_PARAMETER_PACKS);
                    if(ServicableComponent.class.isAssignableFrom(meta.getComponentImplementation())) {
                        ServicableComponent scomp = (ServicableComponent) comp;
                        if(scomp.isStarted()) {
                            scomp.stop();
                        }
                    }
                    if(InitializableComponent.class.isAssignableFrom(meta.getComponentImplementation())) {
                        InitializableComponent dcomp = (InitializableComponent) comp;
                        dcomp.destroy();
                    }
                }
            }
            catch(Exception e) {
                e.printStackTrace();
            }
        }
        started = false;
    }

    /**
     * Registry Ƿ
     */
    public synchronized boolean isStarted() {
        return started;
    }

    /**
     * ʹ Registry װؿ
     * @param impl Ҫص
     * @throws ClassNotFoundException
     */
    public Class loadClass(String impl) throws ClassNotFoundException {
        return loaderRepo.loadClass(impl);
    }

    /**
     *  Registry װؿ·
     * @param url  jar · class Ŀ¼
     */
    public void addLibraryURL(URL url) {
        loaderRepo.addURL(url);
    }

    /**
     * õ Registry ؿ
     * @return
     */
    public ClassLoaderRepository getClassLoaderRepository() {
        return loaderRepo;
    }

    /**
     * עһ
     * @param meta
     * @throws ComponentException
     */
    private synchronized void doRegister(ComponentMeta meta) throws ComponentException {
        logger.info("register component " + meta.getComponentName());
        repo.registerComponent(meta);
        // set registry
        meta.setRegistry(this);
        // doActive by registerComponent
//        doActive(meta);
    }

    private synchronized void doActive(ComponentMeta meta) throws ComponentException{
        // instance the component immediately
        if(ActiveComponent.class.isAssignableFrom(meta.getComponentImplementation())) {
            getComponentInstance(meta.getComponentName());
        }

    }

    /**
     * ʼ
     * @param meta
     * @param comp
     * @throws ComponentException
     */
    void initComponent(ComponentMeta meta, Component comp) throws ComponentException {
        if(InitializableComponent.class.isAssignableFrom(comp.getClass())) {
            InitializableComponent icomp = ((InitializableComponent) comp);
            try {
                logger.info("initializing component " + meta.getComponentName());
                icomp.init();
                logger.info("initialize component " + meta.getComponentName() + ", OK!");
            }
            catch(Exception e) {
                logger.warn("initialize component " + meta.getComponentName() + " failed.", e);
                throw new ComponentException(e);
            }
        }
    }


    /**
     * 
     * @param meta
     * @param comp
     * @throws ComponentException
     */
    void startComponent(ComponentMeta meta, Component comp) throws ComponentException {
        // try to start component
        if(ServicableComponent.class.isAssignableFrom(meta.getComponentImplementation())) {
            ServicableComponent scomp = (ServicableComponent) comp;
            try {
                if(!scomp.isStarted()) {
                    logger.info("starting component " + meta.getComponentName());
                    scomp.start();
                    logger.info("start component " + meta.getComponentName() + ", OK!");
                }
            }
            catch(Exception e) {
                logger.info("start component " + meta.getComponentName() + " failed.", e);
                throw new ComponentException(e);
            }
        }
    }

    /**
     * ֹͣ
     * @param meta
     * @param comp
     * @throws ComponentException
     */
    void stopComponent(ComponentMeta meta, Component comp) throws ComponentException {
        if(comp instanceof ServicableComponent) {
            try {
                ServicableComponent scomp = ((ServicableComponent) comp);
                if(scomp.isStarted()) {
                    logger.info("stoping component " + meta.getComponentName());
                    scomp.stop();
                    logger.info("stop component " + meta.getComponentName() + ", OK!");
                }
            }
            catch(Exception e) {
                logger.warn("stop component " + meta.getComponentName() + " failed.", e);
                throw new ComponentException(e);
            }
        }
    }

    /**
     * 
     * @param meta
     * @param comp
     * @throws org.jfox.ioc.exception.ComponentException
     */
    void destroyComponent(ComponentMeta meta, Component comp) throws ComponentException {
        if(comp instanceof InitializableComponent) {
            try {
                InitializableComponent scomp = ((InitializableComponent) comp);
                logger.info("destroying component " + meta.getComponentName());
                scomp.destroy();
                logger.info("destroy component " + meta.getComponentName() + ", OK!");
            }
            catch(Exception e) {
                logger.warn("destroy component " + meta.getComponentName() + " failed.", e);
                throw new ComponentException(e);
            }
        }
    }

    class Repository {
        /**
         * 洢 ComponentName=>ComponentMeta ӳϵ
         */
        private Map<ComponentName, ComponentMeta> components = new LinkedHashMap<ComponentName, ComponentMeta>();
        /**
         * 洢 Implementation=>counter ӳϵ
         * ṩһǷעᣬԼһע˶ٴβѯ
         */
        private Map<Class, List<ComponentName>> compCounts = new HashMap<Class, List<ComponentName>>();

        Repository() {
        }

        public synchronized void registerComponent(ComponentMeta meta) throws ComponentException {
            ComponentName name = meta.getComponentName();
            if(isRegistered(name)) {
                throw new ComponentException("Component " + name + " has already registered.");
            }
            else { // type singleton check
                if(meta.isTypeSingleton()) {
                    // have register this type
                    if(isRegisteredImplemetation(meta.getComponentImplementation())) {
                        throw new ComponentException(meta + " is type singleton, but this type has already been registered.");
                    }
                }
                else {
                    // עһ TypeSingleton true Ȼעһ TypeSingleton falseҲӦñ
                    if(isRegisteredImplemetation(meta.getComponentImplementation())) {
                        List<ComponentName> names = getComponentNamesByImplemetation(meta.getComponentImplementation());
                        // just only check the last Component
                        ComponentName _name = names.get(names.size() - 1);
                        if(components.get(_name).isTypeSingleton()) {
                            throw new ComponentException("a component of this type has already been registered before " + _name.toString());
                        }
                    }
                }
            }

            List<ComponentName> metas;
            if(isRegisteredImplemetation(meta.getComponentImplementation())) {
                metas = (List<ComponentName>) (compCounts.get(meta.getComponentImplementation()));
            }
            else {
                metas = new ArrayList<ComponentName>();
                compCounts.put(meta.getComponentImplementation(), metas);
            }
            metas.add(name);

            components.put(name, meta);
        }

        public synchronized ComponentMeta unregisterComponent(ComponentName name) throws ComponentException {
            if(isRegistered(name)) {
                ComponentMeta meta = (ComponentMeta) components.remove(name);

                List<ComponentName> names = ((List<ComponentName>) compCounts.get(meta.getComponentImplementation()));
                names.remove(name);

                if(names.isEmpty()) {
                    compCounts.remove(meta.getComponentImplementation());
                }
                return meta;
            }
            return null;
        }

        /**
         *  ComponentName õ ComponentMeta
         *
         * @param name ע
         */
        public synchronized ComponentMeta getComponentMeta(ComponentName name) {
            return (ComponentMeta) components.get(name);
        }

        public boolean isRegistered(ComponentName name) {
            return components.containsKey(name);
        }

        public boolean isRegisteredImplemetation(Class clz) {
            return compCounts.containsKey(clz);
        }

        /**
         * õԸעƣԸע
         *
         * @param type 
         */
        public ComponentName getLastComponentNameByType(Class type) {
            List names = new ArrayList(components.keySet());
            for(int i = names.size() - 1; i >= 0; i--) {
                ComponentName name = (ComponentName) names.get(i);
                if(name.getType().isAssignableFrom(type)) {
                    return name;
                }
            }
            return null;
        }

        /**
         * õԸעƣԸע
         * ڸݲԶѰʱҪõ
         *
         * @param type 
         */
        public synchronized ComponentName getFirstComponentNameByType(Class type) {
            for(ComponentName name : components.keySet()) {
                if(type.isAssignableFrom(name.getType())) {
                    return name;
                }
            }
            return null;
        }

        /**
         * صǰעĿ
         */
        public synchronized int countComponents() {
            return components.size();
        }

        /**
         * صǰע˶͵
         */
        public synchronized int countComponentTypes() {
            return compCounts.size();
        }

        /**
         * õע
         */
        public synchronized List<ComponentName> getAllComponentNames() {
            return Collections.unmodifiableList(new ArrayList<ComponentName>(components.keySet()));
        }

        public synchronized List<ComponentMeta> getAllComponentMetas() {
            return Collections.unmodifiableList(new ArrayList<ComponentMeta>(components.values()));
        }

        /**
         * õԸעԸע
         *
         * @param type 
         */
        public List<ComponentName> getComponentNamesByImplemetation(Class type) {
            if(isRegisteredImplemetation(type)) {
                return compCounts.get(type);
            }
            else {
                return new ArrayList<ComponentName>();
            }
        }

    }

}

