/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */

package org.jfox.ioc.classloader;

import java.io.File;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.WeakHashMap;

import org.jfox.ioc.Constants;
import org.jfox.ioc.logger.Logger;
import org.jfox.ioc.util.FileFilters;
import org.jfox.ioc.util.Files;

/**
 * ioc container's class loader repository
 *
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public class ClassLoaderRepository extends URLClassLoader {
    private Logger logger = Logger.getLogger(ClassLoaderRepository.class
                                             .getName());

    /**
     * class name => class object ʹWeakHashMap֤Classû֮󣬿Ա
     */
    private Map classes = new WeakHashMap();

    private final static String COMMON_LIBS = System.getProperty(Constants.JFOXCO_COMMON_LIBS,
                                                                 System.getProperty(Constants.JFOXCO_HOME) + "/common");

    /**
     * ¼עĸģClassLoader
     */
    private List<LocalClassLoader> localCLs = new ArrayList<LocalClassLoader>();

    /**
     * еClassLoaderװ
     * ֮ǰwaitװأͷ synchronize 
     * Ϊӵ ReentranetLock ߳̿Ҫʹø ClassLoader װ
     */
//    private ReentrantLock lock = new ReentrantLock();

    public ClassLoaderRepository(URL[] urls, ClassLoader parent) {
        super(urls, parent);
        loadExternalLibs();
    }

    /**
     * װ external Ŀ¼µ jar
     */
    private void loadExternalLibs() {
        File libDir = new File(COMMON_LIBS);
        if(!libDir.exists() || libDir.isFile())
            return;
        List jarFiles = Files.listFiles(libDir, FileFilters
                                                .suffixFileFilter(new String[]{"jar", "zip"}));
        for(int i = 0; i < jarFiles.size(); i++) {
            File file = (File) jarFiles.get(i);
            try {
                addURL(file.toURL());
            }
            catch(Exception e) {
                e.printStackTrace();
            }
        }
    }

    public URL[] getURLs() {
        return super.getURLs();
    }

    synchronized void addLocalCL(LocalClassLoader localCL) {
        if(!localCLs.contains(localCL)) {
            localCLs.add(localCL);
        }
    }

    public void removeLocalCL(LocalClassLoader localCL) {
        localCLs.remove(localCL);
    }

    /**
     *  URL  Classpath ·
     *
     * @param url
     */
    public void addURL(URL url) {
        logger.debug("add class path " + url.toString());
        super.addURL(url);
    }

    /**
     *
     * @param name
     * @return
     * @throws ClassNotFoundException
     */
//    public synchronized Class loadClass(String name) throws ClassNotFoundException {

//    }

    /**
     * Ϊ ClassLoaderRepository  LocalClassLoader  parent ClassLoader
     * LocalClassLoader ֱӵԱҪ
     * ȴ ClassLoaderRepository classpath·װ࣬ʧܣ װػ LocalClassLoader
     * Ѿװص
     * @param name
     * @param resolve
     * @return
     * @throws ClassNotFoundException
     */
    protected synchronized Class<?> loadClass(String name, boolean resolve) throws ClassNotFoundException {
        try {
            return super.loadClass(name,resolve);
        }
        catch(ClassNotFoundException e) {
            try {
                return loadClassCached(name);
            }
            catch(ClassNotFoundException e1) {

            }
        }
        throw new ClassNotFoundException(name);

    }

    /**
     * װϵͳԼ common libs е
     *
     */
/*
    synchronized Class loadClassSuper(String name) throws ClassNotFoundException {
        return super.loadClass(name);
    }
*/

    synchronized void cacheClass(Class clz) {
        if(!classes.containsKey(clz.getName())) {
            logger.debug("cache " + clz + ", classloader is "
                         + clz.getClassLoader());
            classes.put(clz.getName(), clz);
        }
    }

    /**
     * װLocalClassLoader Cache
     *
     * @param name
     * @return
     * @throws ClassNotFoundException
     */
    synchronized Class loadClassCached(String name) throws ClassNotFoundException {
        if(!classes.containsKey(name)) {
            throw new ClassNotFoundException(name);
        }
        return (Class) classes.get(name);
    }


    /**
     * Ա LocalClassLoader װ
     */
/*
    private synchronized Class loadClassWidely(String name) throws ClassNotFoundException {
        for(int i = 0; i < localCLs.size(); i++) {
            LocalClassLoader loader = localCLs.get(i);
            try {
                Class clz = loader.loadClass(name);
                return clz;
            }
            catch(ClassNotFoundException e) {

            }
        }
        throw new ClassNotFoundException(name);
    }
*/

    /**
     * Ա LocalClassLoader װ
     */
/*
    synchronized Class loadClassWidelyWithout(String name, LocalClassLoader localCL)
            throws ClassNotFoundException {
        for(int i = 0; i < localCLs.size(); i++) {
            LocalClassLoader loader = localCLs.get(i);
            if(loader != localCL) {
                try {
                    Class clz = loader.loadClassLocal(name);
                    return clz;
                }
                catch(ClassNotFoundException e) {

                }
            }
        }
        throw new ClassNotFoundException(name);
    }
*/

/*
    public boolean tryLock(){
        return lock.tryLock();
    }

    public void lock(){
        lock.lock();
    }

    public void unlock(){
        lock.unlock();
    }

    public ReentrantLock getLock(){
        return lock;
    }

    public void notifyAllCL(){
        for(int i = 0; i < localCLs.size(); i++) {
            LocalClassLoader loader = localCLs.get(i);
            synchronized(loader){
                loader.notifyAll();
            }
        }
    }
*/

    public static void main(String[] args) {

    }
}