/*
 * Copyright (c) 2004 Your Corporation. All Rights Reserved.
 */
package org.jfox.ioc.common;

import org.jfox.ioc.ComponentContext;
import org.jfox.ioc.Registry;
import org.jfox.ioc.annotation.Managable;
import org.jfox.ioc.ext.ActiveComponent;
import org.jfox.ioc.ext.ManagableComponent;

/**
 * @author <a href="mailto:yy.young@gmail.com">Young Yang</a>
 */

public class ShutdownHook extends AbstractService implements ManagableComponent, ActiveComponent{
    private Registry registry = null;
    private Thread shutdownThread = null;

    protected void doInit() throws Exception {
        shutdownThread = new Thread(this);
        shutdownThread.setName(getName());

    }

    protected void doStart() throws Exception {
        Runtime.getRuntime().addShutdownHook(shutdownThread);
    }

    protected void doStop() throws Exception {
        // if shutdown thread is working, can not remove it
        if(!shutdownThread.isAlive()) {
            Runtime.getRuntime().removeShutdownHook(shutdownThread);
        }
    }

    protected void doDestroy() throws Exception {
        shutdownThread = null;
    }

    public void run() {
        logger.info("stopping system.");
        try {
            registry.stop();
        }
        catch(Exception e) {
            e.printStackTrace();
        }

    }

    @Managable
    public void stopSystem() {
        Thread t = new Thread(this);
        t.start();

        try {
            t.join();
//            Thread.sleep(2000L);
//            System.exit(0);
        }
        catch(Exception e){

        }

    }

    public void setComponentContext(ComponentContext ctx) {
        super.setComponentContext(ctx);
        registry = ctx.getRegistry();
    }

    public static void main(String[] args) {

    }
}

