/*
 * Copyright (c) 2004 Your Corporation. All Rights Reserved.
 */
package org.jfox.ioc.connector;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.HashMap;
import java.util.Map;

import org.jfox.ioc.util.Methods;

/**
 * @author <a href="mailto:yy.young@gmail.com">Young Yang</a>
 */

public abstract class AbstractConnectorInvoker implements ConnectorInvoker {
    protected ObjectId clientId;
    protected ConnectorRemote remote;

    protected transient Map<Method,String> methods = new HashMap<Method,String>();

    protected AbstractConnectorInvoker(ObjectId clientId, ConnectorRemote remote) {
        this.clientId = clientId;
        this.remote = remote;
    }

    public ObjectId getClientId() {
        return clientId;
    }

    public final Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
        if(method.equals(Methods.ToString)) {
            return ((ConnectorInvoker) Proxy.getInvocationHandler(proxy)).getClientId().toString();
        }
        else if(method.equals(Methods.HashCode)) {
            return new Integer(clientId.hashCode());
        }
        return doInvoke(proxy,method,args);
    }

    protected String getMethodHash(Method method){
        if(methods == null) {
            methods = new HashMap<Method, String>();
        }
        if(methods.containsKey(method)){
            return methods.get(method);
        }
        else {
            String hash = String.valueOf(Methods.getMethodHash(method));
            methods.put(method,hash);
            return hash;
        }
    }

    protected abstract Object doInvoke(Object proxy, Method method, Object[] args) throws Throwable;

}

