/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */

package org.jfox.ioc.connector;

import java.lang.reflect.InvocationTargetException;
import java.rmi.RemoteException;
import java.util.Date;

import org.jfox.ioc.ComponentContext;
import org.jfox.ioc.Registry;
import org.jfox.ioc.common.AbstractService;
import org.jfox.ioc.ext.ActiveComponent;

/**
 * Զ̽ĳ
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public abstract class AbstractConnectorRemote extends AbstractService implements ConnectorRemote, ActiveComponent {

    /**
     * ConnectorHandler 
     */
    private transient HandlerManager handlerManager = HandlerManager.getInstance();

    private int port = 0;

    protected transient Registry registry;

    public Object invoke(Invocation invocation) throws RemoteException, Exception {
        try {
            return handlerManager.execute(invocation);
        }
        catch(Throwable e) {
            Exception ex = (Exception) e;
            if(e instanceof InvocationTargetException) {
                Throwable t = ((InvocationTargetException) e).getTargetException();
                if(t instanceof Exception) {
                    ex = (Exception) t;
                }
                else {
                    ex = new Exception(t);
                }
            }
            logger.warn(ex.getMessage(), ex);
            if(ex instanceof RuntimeException) {
                throw new RemoteException(((RuntimeException) ex).getMessage(), ex);
            }
            else {
                throw ex;
            }
        }
    }

    public void ping() throws RemoteException {
        logger.debug("ping pang - " + new Date());
    }

    public void setPort(int port) {
        this.port = port;
    }

    public int getPort() {
        return port;
    }

    public void setComponentContext(ComponentContext ctx) {
        super.setComponentContext(ctx);
        registry = context.getRegistry();
    }

    public void run() {
        while(isStarted()) {
            try {
                Thread.sleep(30 * 60 * 1000L);
                ping();
            }
            catch(Exception e) {

            }
        }
    }

    public static void main(String[] args) {

    }
}
