/*
 * Copyright (c) 2004 Your Corporation. All Rights Reserved.
 */
package org.jfox.ioc.connector;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jfox.ioc.common.AbstractComponent;
import org.jfox.ioc.ext.ActiveComponent;
import org.jfox.ioc.ext.SingletonComponent;
import org.jfox.ioc.util.Classes;
import org.jfox.ioc.util.Methods;

/**
 * ÿModuleԼModuleʵһ HandlerSupport ࣬
 * ΪҪõǰ߳װΪHandlerװ
 *
 * @author <a href="mailto:yy.young@gmail.com">Young Yang</a>
 */

public abstract class AbstractHandler extends AbstractComponent implements Handler, ActiveComponent, SingletonComponent {

    protected Container container;

    protected Map<String, Method> methodHashers = new HashMap<String, Method>();

    public AbstractHandler(Container container) {
        this.container = container;
        if(!container.getHandlerClass().isAssignableFrom(this.getClass())){
            throw new RuntimeException("container " + container + " 's handler must be " + container.getHandlerClass().getName() + ", can not be " + this.getClass().getName());
        }
        methodHashers.putAll(introspectorMethods());
    }

    /**
     * Handler  Container
     */
    public AbstractHandler(){
        if(!(this instanceof Container)){
            throw new IllegalArgumentException("Handler " + this.getClass().getName() + " not implement " + Container.class.getName());
        }
        container = (Container)this;
    }

    public Object execute(Invocation invocation) throws Throwable {
        if(!isInitialized()) {
            throw new Exception("Handler not started.");
        }
        String hash = invocation.getMethodHash();
        Method method = getMethodByHash(hash);
        invocation.setMethod(method);
        if(method == null) {
            throw new NoSuchMethodException(hash);
        }

        ClassLoader oldCl = Thread.currentThread().getContextClassLoader();
        /**
         * Ҫó Module  LocalClassLoader
         *  Module ûԤصҲ
         */
        ClassLoader cl = this.getClass().getClassLoader();
        Thread.currentThread().setContextClassLoader(cl);
        try {
            return doExecute(invocation);
        }
        finally {
            Thread.currentThread().setContextClassLoader(oldCl);
        }
    }

    /**
     * Containerķ
     * @param invocation
     * @return
     * @throws Throwable
     */
    protected Object doExecute(Invocation invocation) throws Throwable {
        logger.debug(invocation);
        return invocation.getMethod().invoke(container, invocation.getArgs());

    }

    protected void doInit() throws Exception {
        HandlerManager.getInstance().registerHandler(getInvocationClass(), this);
    }

    protected void doDestroy() throws Exception {
        HandlerManager.getInstance().removeHandler(getInvocationClass());
    }

    public Container getContainer() {
        return container;
    }

    /**
     * Containerнӿڵķ
     */
    protected Map<String, Method> introspectorMethods() {
        Map<String, Method> methodHashers = new HashMap<String, Method>();
        // cache нӿڵķ
        List<Class> clzes = Classes.getAllInterfaces(getContainer().getClass());
        for(Class interf : clzes) {
            List<Method> methods = Classes.introspectMethods(interf);
            for(Method method : methods) {
                methodHashers.put(String.valueOf(Methods.getMethodHash(method)), method);
            }
        }
        return methodHashers;
    }

	public Method getMethodByHash(String hashId) {
		return methodHashers.get(hashId);
	}

}

