/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */

package org.jfox.ioc.connector;

import java.rmi.MarshalledObject;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jfox.ioc.common.AbstractService;
import org.jfox.ioc.util.Marshaller;
import org.jgroups.Address;
import org.jgroups.Channel;
import org.jgroups.JChannel;
import org.jgroups.MembershipListener;
import org.jgroups.Message;
import org.jgroups.View;
import org.jgroups.blocks.GroupRequest;
import org.jgroups.blocks.MessageDispatcher;
import org.jgroups.blocks.RequestHandler;
import org.jgroups.stack.IpAddress;

/**
 * JGroupϢɷӦHandler
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public class ClusterServer extends AbstractService implements RequestHandler,MembershipListener {

	public final static String CHANNEL_NAME = "__JFOX_CLUSTER__";

	/**
	 * JGroup Properties
	 */
	protected String jgroupProps =
			"UDP(mcast_addr=228.1.2.3;mcast_port=45566;ip_ttl=32):" +
			"PING(timeout=3000;num_initial_members=6):" +
			"FD(timeout=3000):" +
			"VERIFY_SUSPECT(timeout=1500):" +
			"pbcast.NAKACK(gc_lag=10;retransmit_timeout=600,1200,2400,4800):" +
			"UNICAST(timeout=600,1200,2400,4800):" +
			"pbcast.STABLE(desired_avg_gossip=10000):" +
			"FRAG:" +
			"pbcast.GMS(join_timeout=5000;join_retry_timeout=2000;" +
			"shun=true;print_local_addr=true)";

	/**
	 * ýڵϢ
	 */
	private ServerNode theNode = ServerNode.THE_NODE;

	/**
	 * Ⱥеڵ
	 * key=ip:udp_port
	 */
	private Map<String, ServerNode> clusterNodes = new HashMap<String, ServerNode>();

	// channel
	private JChannel channel = null;

	// message dispatcher
	private MessageDispatcher disp = null;

	public ClusterServer() {
	}

	public String getChannelName() {
		return CHANNEL_NAME;
	}

	public String getJGroupProps() {
		return jgroupProps;
	}

	public void setJGroupProps(String jgroupProps) {
		this.jgroupProps = jgroupProps;
	}

	/**
	 * ȡõǰȺϵͳеĽڵ
	 */
	public synchronized List<ServerNode> getClusterNodes() {
		return Collections.unmodifiableList(new ArrayList<ServerNode>(clusterNodes.values()));
	}
	
	public Object handle(Message msg) {
		// £wrappedObject  MarshalledObject cast
		Object wrappedObject = msg.getObject();
		String debugMsg = "handle message " + wrappedObject;
		try {
			if(wrappedObject instanceof MarshalledObject) {
				wrappedObject = ((MarshalledObject) wrappedObject).get();
				debugMsg += wrappedObject.toString();
			}
		}
		catch(Exception e) {
			e.printStackTrace();
		}
		logger.debug(debugMsg);

		if(wrappedObject instanceof ServerNode) { // ͬ,JGroupsԶͬView
			// ռȺڵ,һµĽڵ
			ServerNode node = (ServerNode) wrappedObject;
            collectServerNode(node);
		}
		else if(wrappedObject instanceof ClusterInvocation) {
            // ͬBean
			ClusterInvocation invocation = (ClusterInvocation) wrappedObject;
            // ClusterInvocation ȡ ServerNode½ڵ㣬ռýڵ
            ServerNode sourceNode = invocation.getClusterNode();
            collectServerNode(sourceNode);
            try {
				HandlerManager.getInstance().execute(invocation);
			}
			catch(Throwable e) {
				logger.warn("cluster synchronize error!", e);
			}
		}
		return null;
	}

	public void viewAccepted(View view) {
		try {
			logger.debug("viewAccepted: " + view);
			// View Members  ClusterNode ͬ
			Map<String, ServerNode> newClusterNodes = new HashMap<String, ServerNode>();
			List list = view.getMembers();
			for(Object addr : list) {
				IpAddress ipAddress = (IpAddress) addr;
				String key = ipAddress.getIpAddress().getHostAddress() + ":" + ipAddress.getPort();
				if(ipAddress.getIpAddress().getHostAddress().equals(theNode.getIp())
				   && (ipAddress.getPort() == theNode.getJgport())) {
					continue;
				}
				// Ѿвŵ node
				if(clusterNodes.containsKey(key)) {
					newClusterNodes.put(key, clusterNodes.get(key));
				}
			}
			clusterNodes = newClusterNodes;
			//пһµĽڵ㣬ԼServerNodeȥԱܹעᵽµĽڵ
			Message myNodeMessage = new Message(null, null, theNode);
			//˴ʹ GET_ALL
			disp.castMessage(null,myNodeMessage,GroupRequest.GET_NONE,0);
		}
		catch(Exception e) {
			logger.warn(e.getMessage(), e);
		}

	}

	public void suspect(Address suspected_mbr) {
        logger.debug("suspect: " + suspected_mbr);
	}

	public void block() {
		logger.debug("block invoked.");
	}

	/**
	 * ȺзҪͬInvocationϢ
	 *
	 * @param invocation
	 * @throws Exception
	 */
	public void cast(Invocation invocation) throws Exception {
		//ֻClusteableInvocationŷ
		if(invocation != null && (invocation instanceof ClusterInvocation)) {
//			((ClusterInvocation)invocation).getInvocation().setMethod(null);
			MarshalledObject mobj = Marshaller.marshall(invocation);
			Message msg = new Message(null, null, mobj);
//			channel.send(msg);
			//GroupRequest.GET_NONE 첽ģʽȴReponseList
			disp.castMessage(null,msg, GroupRequest.GET_ALL,0);
		}
	}

	protected void doInit() throws Exception {
		this.channel = new JChannel(jgroupProps);
		channel.setOpt(Channel.GET_STATE_EVENTS, new Boolean(true));
		channel.setOpt(Channel.AUTO_RECONNECT, new Boolean(true));
		channel.setOpt(Channel.AUTO_GETSTATE, new Boolean(true));
		channel.setOpt(Channel.LOCAL, new Boolean(false));

		logger.debug("Channel " + CHANNEL_NAME + " created.");
	}

	protected void doStart() throws Exception {
		channel.connect(CHANNEL_NAME);
		logger.info("Channel " + CHANNEL_NAME + " connected.");
		theNode.setJgport(((IpAddress) channel.getLocalAddress()).getPort());
		disp = new MessageDispatcher(channel, null, this, this, true);
		logger.error("run here impossible.");
	}

	protected void doStop() throws Exception {
		disp.stop();
		channel.close();
	}

	protected void doDestroy() throws Exception {
		channel = null;
		disp = null;
	}

	public void run() {
		try {
		}
		catch(Exception e){
			logger.error(e.getMessage(),e);
		}
	}

    /**
     * ռڵϢ
     * @param node
     */
    private void collectServerNode(ServerNode node){
        String key = node.getIp() + ":" + node.getJgport();
        if(!node.equals(theNode) && !clusterNodes.containsKey(key)) {
            logger.info("collect new ServerNode: " + key);
            clusterNodes.put(key, node);
        }
    }

    public static void main(String[] args) {

	}
}
