/*
 * Copyright (c) 2004 Your Corporation. All Rights Reserved.
 */
package org.jfox.ioc.connector;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ObjectInputStream;
import java.io.PrintStream;
import java.net.Socket;
import java.net.URI;
import java.net.URISyntaxException;
import java.rmi.MarshalledObject;
import java.util.HashMap;
import java.util.Map;
import javax.rmi.PortableRemoteObject;

/**
 * @author <a href="mailto:yy.young@gmail.com">Young Yang</a>
 */

public class ConnectorHelper {

    /**
     * cache connector stub
     */
    private static Map connectors = new HashMap();

    public static ConnectorRemote lookupConnector(String providerURL) {
        URI uri;
        try {
            uri = new URI(providerURL);
        }
        catch(URISyntaxException e){
            throw new RuntimeException(e);
        }
        String host = uri.getHost();
        int port = uri.getPort();
        String protocol = uri.getScheme();
        String key = providerURL;
        if(connectors.containsKey(key)) { // return from cache
            ConnectorRemote remote = (ConnectorRemote) connectors.get(key);
            try {
                remote.ping();
                return remote;
            }
            catch(Throwable e) {
                connectors.remove(key);
                e.printStackTrace();
            }
        }

        try {
	        ConnectorRemote stub = lookupConnector(host,port,protocol);
            connectors.put(key, stub);
            return stub;
        }
        catch(Throwable e) {
            e.printStackTrace();
            connectors.remove(key);
            return null;
        }
    }

	public static ConnectorRemote lookupConnector(String host, int port, String protocol) throws Exception {
		Socket csocket = new Socket(host, port);
		// try get the remote of the protocol
		PrintStream out = new PrintStream(new BufferedOutputStream(csocket.getOutputStream()));
		out.println(protocol);
		out.flush();

		ObjectInputStream in = new ObjectInputStream(new BufferedInputStream(csocket.getInputStream()));
		Object obj = in.readObject();
		csocket.close();
		ConnectorRemote stub;
		if(obj instanceof MarshalledObject) {
			MarshalledObject mobj = (MarshalledObject) PortableRemoteObject.narrow(obj, MarshalledObject.class);
			stub = (ConnectorRemote) mobj.get();
		}
		else {
			stub = (ConnectorRemote) PortableRemoteObject.narrow(obj, ConnectorRemote.class);
		}
		stub.ping();
		return stub;
	}

    public static void main(String[] args) {

    }
}

