/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */

package org.jfox.ioc.connector;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.ObjectOutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;
import java.net.InetAddress;
import java.rmi.MarshalledObject;
import java.util.List;
import java.util.ArrayList;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.jfox.ioc.common.AbstractService;
import org.jfox.ioc.ext.ActiveComponent;
import org.jfox.ioc.ext.SingletonComponent;
import org.jfox.ioc.util.Marshaller;

/**
 *  1099 ˿ڣ ConnectorService stub
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public class ConnectorServer extends AbstractService implements ActiveComponent, SingletonComponent {
    /**
     * ConnectorServerĶ˿ںţڷ connector stub
     */
    protected static int DEFAULT_PORT = 1099;

    protected ServerSocket ssocket = null;

	/**
	 * ǰڵ
	 */
    public static final ServerNode theNode = ServerNode.THE_NODE;

	/**
	 * ClusterServerãCluste ServerҪ ConnectorRemote аװ
	 */
	protected static ClusterServer clusterServer = null;

	private ExecutorService pool = Executors.newCachedThreadPool();

    public ConnectorServer() throws Exception {
        this(DEFAULT_PORT);
    }

    public ConnectorServer(int port) throws Exception {
        InetAddress address = InetAddress.getLocalHost();
        String ip = address.getHostAddress();
        String hostName = address.getCanonicalHostName();
	    theNode.setIp(ip);
	    theNode.setPort(port);
        theNode.setHostName(hostName);
    }

    public int getPort() {
        return theNode.getPort();
    }

    public void setPort(int port) {
        theNode.setPort(port);
    }

    protected void doStart() throws Exception {
	    if(context.getRegistry().isRegisteredImplemetation(ClusterServer.class)) {
		    // ע˼Ⱥ˼Ⱥ
		    try {
			    clusterServer = (ClusterServer) context.getRegistry().getComponentInstance(ClusterServer.class);
		    }
		    catch(Exception e) {
			    e.printStackTrace();
		    }
	    }
//	    theNode.setConnectors(connectorManager.getConnectors());
	    Thread t = new Thread(this, this.getClass().getName());
	    t.start();
	    Thread.sleep(50);
//	    t.join();
    }

	public static ClusterServer getClusterServer() {
		return clusterServer;
	}

	protected void doStop() throws Exception {
        ssocket.close();
    }

    protected void doInit() throws Exception {
        ssocket = new ServerSocket(theNode.getPort());
        ssocket.setReuseAddress(true);
    }

    protected void doDestroy() throws Exception {
        ssocket = null;
    }

    public void run() {
        while(isStarted()) {
            Socket socket = null;
            try {
                socket = ssocket.accept();
                socket.setReuseAddress(true);
            }
            catch(SocketException e) {
//                e.printStackTrace();
            }
            catch(IOException e){
                e.printStackTrace();
            }
            if(socket != null) {
	            pool.execute(new Sender(socket));
            }
        }
    }

    protected ConnectorRemote getConnectorRemote(String protocol){
        ConnectorRemote remote = theNode.getConnector(protocol);
	    if(clusterServer != null){
		    // ע˼Ⱥ˼Ⱥ
		    try {
			    List<ServerNode> nodes = clusterServer.getClusterNodes();
			    List<ConnectorRemote> remotes = new ArrayList<ConnectorRemote>(nodes.size());
			    for(ServerNode node : nodes ){
				    remotes.add(node.getConnector(protocol));
			    }
			    remote = new ClusterableConnectorRemote(remote,remotes.toArray(new ConnectorRemote[remotes.size()]));
		    }
		    catch(Exception e){
			    e.printStackTrace();
		    }
	    }
        return remote;
    }

	private class Sender implements Runnable {
		private Socket socket;

		public Sender(Socket socket) {
			this.socket = socket;
		}

		public void run() {
			try {
				BufferedReader in = new BufferedReader(new InputStreamReader((socket.getInputStream())));
				String protocol = in.readLine();
				ConnectorRemote remote = getConnectorRemote(protocol);
				MarshalledObject mobj = Marshaller.marshall(remote);
				ObjectOutputStream out = new ObjectOutputStream(new BufferedOutputStream(socket.getOutputStream()));
				out.writeObject(mobj);
				out.flush();
				socket.close();
			}
			catch(Exception e) {
				logger.warn(e.getMessage(), e);
			}
		}
	}

    public static void main(String[] args) {

    }
}
