/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */

package org.jfox.ioc.connector;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jfox.ioc.Registry;
import org.jfox.ioc.common.AbstractComponent;
import org.jfox.ioc.ext.ActiveComponent;
import org.jfox.ioc.ext.ManagableComponent;
import org.jfox.ioc.ext.SingletonComponent;

/**
 * ܵHandlerЭHandlerעᵽConnectorRemoteֻHandler
 *
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public class HandlerManager extends AbstractComponent implements ActiveComponent,ManagableComponent, SingletonComponent {
    /**
     * invocation class => Handler
     */
    private Map<Class, Handler> handlers = new HashMap<Class, Handler>();

    protected Registry registry;

    private final static HandlerManager ME = new HandlerManager();

    private HandlerManager() {
    }

    public static HandlerManager getInstance(){
        return ME;
    }

    /**
     * עһӦЭӦ
     * ConnectorServiceݲͬӦЭòͬӦ
     * һConnectorֶ֧ӦЭĵ
     * 磺ȿӦEJBҲӦ JMS 
     * @param invocationType must be a Invocation class
     * @param handler
     */
    public synchronized void registerHandler(Class invocationType, Handler handler) {
        if(Invocation.class.isAssignableFrom(invocationType)){
            handlers.put(invocationType, handler);
        }
    }

    public synchronized void removeHandler(Class invocationType) {
        handlers.remove(invocationType);
    }

    public synchronized Handler getHandler(Class invocationType) {
        return handlers.get(invocationType);
    }

    public List<Handler> listHandlers() {
        return Collections.unmodifiableList(new ArrayList<Handler>(handlers.values()));
    }

    public Object execute(Invocation invocation) throws Throwable {
        logger.debug(invocation);
        Class invocationClass = invocation.getClass();
        Handler handler = getHandler(invocationClass);
        if(handler == null) {
            throw new Exception("no handler response for invocation " + invocation);
        }
        Object result = handler.execute(invocation);

	    try {
		    //ClusterInvocation,Žͬ,ΪClusterInvocationͬĵ
		    if(!(invocation instanceof ClusterInvocation) && invocation.isClustable()){
			    try {
				    // ͨ IoC õConnectorServer
//				    ConnectorServer connectorServer = (ConnectorServer) context.getRegistry().getComponentInstance(ConnectorServer.class);
				    ClusterServer clusterServer = ConnectorServer.getClusterServer();
				    if(clusterServer != null){ // clusterServer 
					    ClusterInvocation clusterInvocation = new ClusterInvocation(invocation,ServerNode.THE_NODE);
					    clusterServer.cast(clusterInvocation);
				    }
			    }
			    catch(Exception e) {
				    logger.warn("cluster cast error!",e);
			    }
		    }
	    }
	    finally{
		    return result;
	    }

    }

    protected void doInit() throws Exception {
        registry = context.getRegistry();
    }

    protected void doDestroy() throws Exception {
    }
}
