/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */
package org.jfox.ioc.connector.jrmps;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import org.jfox.ioc.connector.ServerNode;
import org.jfox.ioc.connector.jrmp.JRMPConnectorRemote;
import org.jfox.ioc.ext.ManagableComponent;

/**
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public class JRMPSConnectorRemote extends JRMPConnectorRemote implements ManagableComponent {

    private String SSLKeyFile = "jfox.keystore";
    private String SSLTrustKeyFile = "jfox.truststore";
    private String password = "jfox.org";

    private JRMPSServerSocketFactory serverFactory = null;
    private JRMPSClientSocketFactory clientFactory = null;

    protected void doInit() throws Exception {
        super.doInit();
        clientFactory = new JRMPSClientSocketFactory(SSLTrustKeyFile, password);
        serverFactory = new JRMPSServerSocketFactory(SSLKeyFile, password);
    }

    /**
     * export this invoker and set container's invoker to this stub
     *
     * @throws Exception
     */
    protected void doStart() throws Exception {
        UnicastRemoteObject.exportObject(this, getPort(), clientFactory, serverFactory);
        // register this Connector
	    ServerNode.THE_NODE.registerConnector(getProtocol(), this);

        // һ̣߳ÿһʱ ping һ£ֹ rmi ߳
        Thread pingThread = new Thread(this, getClass().getName());
        pingThread.setPriority(Thread.MIN_PRIORITY);
        pingThread.start();
    }

    protected void doStop() throws Exception {
	    ServerNode.THE_NODE.removeConnector(getProtocol());
        UnicastRemoteObject.unexportObject(this, true);
    }

    protected synchronized void doDestroy() throws Exception {
        super.doDestroy();
        serverFactory = null;
        clientFactory = null;
    }

    public String getProtocol() throws RemoteException {
	    // jrmp_ssl
        return "JRMPS";
    }

    public void setSSLKeyFile(String SSLKeyFile) {
        this.SSLKeyFile = SSLKeyFile;
    }

    public void setSSLTrustKeyFile(String SSLTrustKeyFile) {
        this.SSLTrustKeyFile = SSLTrustKeyFile;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public String getSSLKeyFile() {
        return SSLKeyFile;
    }

    public String getSSLTrustKeyFile() {
        return SSLTrustKeyFile;
    }

    public static void main(String[] args) {

    }
}