/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */

package org.jfox.ioc.deploy;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.jfox.ioc.ComponentMeta;
import org.jfox.ioc.deployment.Descriptor;
import org.jfox.ioc.util.XMLUtils;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public class ComponentDescriptor implements Descriptor {
    private String desc;
    private String implementation;
    private String name = "";
    private boolean singleton = ComponentMeta.DEFAULT_SINGLETON;
    private boolean typeSingleton = ComponentMeta.DEFAULT_TYPE_SINGLETON;
    private ConstructorDescriptor constructor;
    private List properties = new ArrayList();

    private ComponentModuleDescriptor componentModuleDescriptor;

    public ComponentDescriptor(ComponentModuleDescriptor componentModuleDescriptor) {
        this.componentModuleDescriptor = componentModuleDescriptor;
    }

    public void processXML(Node node) throws ComponentDescriptionException {
        setDescription(XMLUtils.getChildNodeValueOf(node, "description"));
        setName(XMLUtils.getChildNodeValueOf(node, "name"));
        setImplementation(XMLUtils.getChildNodeValueOf(node, "implementation"));
        setSingleton(XMLUtils.getChildNodeValueOf(node, "singleton"));
        setTypeSingleton(XMLUtils.getChildNodeValueOf(node, "type-singleton"));
        Node constructorNode = XMLUtils.getChildNodeOf(node, "constructor");
        if(constructorNode != null) {
            ConstructorDescriptor cd = new ConstructorDescriptor();
            cd.processXML(constructorNode);
            setConstructorDescriptor(cd);
        }

        Iterator it = XMLUtils.getElementsByTagName((Element) node, "property");
        while(it.hasNext()) {
            Node propertyNode = (Node) it.next();
            PropertyDescriptor pd = new PropertyDescriptor();
            pd.processXML(propertyNode);
            addPropertyDescriptor(pd);
        }
    }

    public String getDescription() {
        return desc;
    }

    public void setDescription(String desc) {
        this.desc = desc;
    }

    public String getImplementation() {
        return implementation;
    }

    public void setImplementation(String componentImplementation) {
        this.implementation = componentImplementation;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        if(name != null) this.name = name;
    }

    public boolean isSingleton() {
        return singleton;
    }

    public void setSingleton(String singleton) {
        if(singleton != null && !singleton.equals("")) {
            if(singleton.equals("true") || singleton.equals("false")) {
                this.singleton = Boolean.valueOf(singleton).booleanValue();
            }
        }
    }

    public boolean isTypeSingleton() {
        return typeSingleton;
    }

    public void setTypeSingleton(String typeSingleton) {
        if(typeSingleton != null && !typeSingleton.equals("")) {
            if(typeSingleton.equals("true") || typeSingleton.equals("false")) {
                this.typeSingleton = Boolean.valueOf(typeSingleton).booleanValue();
            }
        }
    }

    public int getPriority() {
        return componentModuleDescriptor.getPriority();
    }

    public ConstructorDescriptor getConstructorDescriptor() {
        return constructor;
    }

    public void setConstructorDescriptor(ConstructorDescriptor constructor) {
        this.constructor = constructor;
    }

    public void addPropertyDescriptor(PropertyDescriptor pd) {
        properties.add(pd);
    }

    public PropertyDescriptor[] getPropertyDescriptors() {
        return (PropertyDescriptor[]) properties.toArray(new PropertyDescriptor[properties.size()]);
    }

    public String getModuleDescription() {
        return componentModuleDescriptor.getDescription();
    }

    public static void main(String[] args) {

    }

}

