/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */

package org.jfox.ioc.deploy;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.jfox.ioc.ComponentMeta;
import org.jfox.ioc.deployment.Descriptor;
import org.jfox.ioc.util.XMLUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public class ComponentModuleDescriptor implements Descriptor {

	private List components = new ArrayList();
	private String description;
	// default priority
	private int priority = ComponentMeta.DEFAULT_PRIORITY;

	private List<String> publicClassList = new ArrayList<String>();

	public ComponentModuleDescriptor(Document document) throws ComponentDescriptionException {
		this.processXML(document.getDocumentElement());
	}

	/**
	 * node is the Document element
	 *
	 * @param node
	 * @throws ComponentDescriptionException
	 */
	public void processXML(Node node) throws ComponentDescriptionException {
		setDescription(XMLUtils.getChildNodeValueOf(node, "description"));
		setPriority(XMLUtils.getChildNodeValueOf(node, "priority"));

		/**
		 * public classes ҪԤװأύ ClassLoader Ĺ
		 */
		Node publicClassesElement = XMLUtils.getChildNodeOf(node, "public-classes");
		if(publicClassesElement != null) {
			Iterator pubClasses = XMLUtils.getElementsByTagName((Element) publicClassesElement, "class");
			while(pubClasses.hasNext()) {
				Element pubClass = (Element) pubClasses.next();
				String pubClassName = XMLUtils.getValueOf(pubClass);
				addPublicClassName(pubClassName);
			}
		}

		Iterator components = XMLUtils.getElementsByTagName((Element) node, "component");
		while(components.hasNext()) {
			Element component = (Element) components.next();
			ComponentDescriptor componentDesc = new ComponentDescriptor(this);
			componentDesc.processXML(component);
			addComponent(componentDesc);
		}
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public int getPriority() {
		return priority;
	}

	public void setPriority(String priority) {
		try {
			this.priority = Integer.parseInt(priority);
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}

	void addComponent(ComponentDescriptor component) {
		components.add(component);
	}

	public ComponentDescriptor[] getComponentDescriptors() {
		return (ComponentDescriptor[]) components.toArray(new ComponentDescriptor[components.size()]);
	}

	public void addPublicClassName(String className) {
		publicClassList.add(className);
	}

	public String[] getPublicClasses() {
		return publicClassList.toArray(new String[publicClassList.size()]);
	}

	public static void main(String[] args) {

	}
}

