/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */

package org.jfox.ioc.deploy;

import java.io.File;
import java.io.FileFilter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.jfox.ioc.ComponentContext;
import org.jfox.ioc.ext.ServicableComponent;
import org.jfox.ioc.logger.Logger;
import org.jfox.ioc.util.FileFilters;

/**
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public class DeployMonitor implements ServicableComponent, Runnable {

    private final static String DEFAULT_MODULE_DIR = "./modules";
    /**
     * ģĿ¼ºиļģ齫ԣ
     */
    private final static String IGNORE_FLAG_FILENAME = ".ignore";
    /**
     * ԵģĿ¼ЩĿ¼ģĿ¼
     */
    private final static String[] IGNORE_MODULE_DIR = new String[]{".","CVS"};

    private boolean started = false;

    // Ĭʹ modules Ŀ¼ΪĿ¼
    private String moduleDir = DEFAULT_MODULE_DIR;
    private ComponentDeployer deployer = null;
    // Modue dir => Module
    private Map<File,ComponentModule> deployments = new HashMap<File,ComponentModule>();

    // module dirs that faild to deployed
    private List failedModules = new ArrayList();

    private static Logger logger = Logger.getLogger(DeployMonitor.class.getName());

    public DeployMonitor(ComponentDeployer deployer) {
        this.deployer = deployer;
    }
    
    public DeployMonitor(ComponentDeployer deployer, String componentsDir) {
        this.deployer = deployer;
        if(componentsDir != null && !componentsDir.equals("")) {
            this.moduleDir = componentsDir;
        }
    }

    /**
     * start to monitor the Component directory
     *
     * @throws Exception
     */
    public void start() throws Exception {
        started = true;
        Thread t = new Thread(this, "DeployMonitor");
        t.setPriority(Thread.MIN_PRIORITY);
        t.start();
    }

    /**
     * stop monitor
     *
     * @throws Exception
     */
    public void stop() throws Exception {
        started = false;
    }


    public void run() {
        File dir = new File(moduleDir);
        if(!dir.exists()) {
            logger.warn("the modules directory \"" + dir.toString() + "\" not exist, deploy monitor stop.");
            return;
        }

        while(isStarted()) {
//      List moduleDirs = new ArrayList(Arrays.asList(dir.listFiles(Files.directoryFileFilter(new String[]{"lib"}))));
            List<File> moduleDirs = new ArrayList<File>(Arrays.asList(dir.listFiles(FileFilters.and(new FileFilter[]{
                FileFilters.directoryFileFilter(),
                FileFilters.not(FileFilters.prefixFileFilter(IGNORE_MODULE_DIR))}))));

            Map<File,ComponentModule> modules = new HashMap<File,ComponentModule>(moduleDirs.size());
            for(Iterator it = moduleDirs.iterator(); it.hasNext();) {
                File moduleDir = (File) it.next();
                if(deployments.containsKey(moduleDir)) { // deployed
                    it.remove();
                    modules.put(moduleDir, deployments.remove(moduleDir));
                }
            }

            // the rest of the deployments must be undeployed
            for(Iterator it = deployments.values().iterator(); it.hasNext();) {
                ComponentModule componentModule = (ComponentModule) it.next();
                logger.info("undeploying module " + componentModule + " ...");
                try {
                    undeploy(componentModule);
                    logger.info("undeploy module " + componentModule + ", OK!");
                }
                catch(DeployException e) {
                    logger.info("undeploy module " + componentModule + " failed.", e);
                }
            }
            deployments.clear();

            // load newModule Dirs
            List<ComponentModule> newModules = new ArrayList<ComponentModule>(moduleDirs.size());
            for(int i = 0; i < moduleDirs.size(); i++) {
                File moduleDir =  moduleDirs.get(i);

                File ignoreFlag = new File(moduleDir,IGNORE_FLAG_FILENAME);
                if(ignoreFlag.exists()){ // ںԱ־ļԸModule
                    logger.info("ignore module " + moduleDir.toString());
                }
                else if(failedModules.contains(moduleDir)) {
                    logger.warn("module " + moduleDir.toString() + " has been deployed failed, skip!!!");
                }
                else {
                    logger.info("parsing module " + moduleDir.toString() + " ...");

                    try {
                        ComponentModule componentModule = new ComponentModule(moduleDir, deployer.getRegistry().getClassLoaderRepository());
                        logger.debug("parse module " + moduleDir.toString() + ", OK!");
                        newModules.add(componentModule);
                        failedModules.remove(componentModule.getPackage());
                    }
                    catch(ComponentDescriptionException e) {
                        failedModules.add(moduleDir);
                        logger.warn("parse module " + moduleDir.toString() + " failed!", e);
                    }
                }
            }
            moduleDirs.clear();
            // sort by priority
            Collections.sort(newModules);
            for(int i = 0; i < newModules.size(); i++) {
                ComponentModule componentModule = (ComponentModule) newModules.get(i);
                if(failedModules.contains(componentModule.getPackage())) {
                    logger.warn("module " + componentModule.getPackage().toString() + " has been deployed failed, skip!!!");
                }
                else {
                    try {
                        logger.info("deploying module " + componentModule + " ...");
                        deployer.deploy(componentModule);
                        logger.debug("deploy module " + componentModule + ", OK!");
                        modules.put(componentModule.getPackage(), componentModule);
                        failedModules.remove(componentModule.getPackage());
                    }
                    catch(DeployException e) {
                        failedModules.add(componentModule.getPackage());
                        logger.warn("deploy module " + componentModule + " failed.", e);
                    }
                }
	            try {
		            //һģ֮Ϣ250ms
	                Thread.sleep(250);
	            }
	            catch(Exception e){
	            	e.printStackTrace();
                }
            }
            newModules.clear();
            // set the new deployments
            deployments = modules;
            try {
                Thread.sleep(5000L);
            }
            catch(InterruptedException e) {
                logger.warn(e);
            }
        }
    }

    private void undeploy(ComponentModule componentModule) throws DeployException {
        deployments.remove(componentModule);
        deployer.undeploy(componentModule);
    }

    public boolean isStarted() {
        return started;
    }

    public File[] getDeployedModules() {
        return (File[]) deployments.keySet().toArray(new File[deployments.size()]);
    }

    public void setComponentContext(ComponentContext ctx) {
    }

    public static void main(String[] args) {

    }
}