/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */

package org.jfox.ioc.factory;

import java.lang.reflect.Constructor;
import java.lang.reflect.Modifier;

import org.jfox.ioc.Component;
import org.jfox.ioc.ComponentName;
import org.jfox.ioc.Registry;
import org.jfox.ioc.depend.Dependency;
import org.jfox.ioc.exception.ComponentException;
import org.jfox.ioc.ext.InstantiatedComponent;
import org.jfox.ioc.util.Classes;

/**
 * עʱṩĲʵע
 * <br>
 * עֻ˹ϵͳԶʹøComponentFactory
 * עʱΪ nullϵͳԶʹһʱҪֻһ
 * Ҫʹ޲ĹôעʱĲӦΪ Parameter[0]
 *
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public class ConstrComponentFactory extends ComponentFactory {
    /**
     * б
     */
    private Dependency[] constrParams;
    /**
     * 
     */
    private Class implementation;
    /**
     * עᵽ Registry е
     */
    private ComponentName compName;
    /**
     * IoC ע
     */
    private Registry registry;

    /**
     * Ƿʵжѭ
     */
    private boolean instantiating = false;
    /**
     * 
     */
    private Constructor constructor;

    /**
     * ʹעƺ͹йComponentFactory
     * ComponentNameа
     * ʹָComponentNameעʱʼComponentFactory
     *
     * @param name ע
     * @param constrParams 
     */
    public ConstrComponentFactory(ComponentName name, Class implementation, Dependency[] constrParams) throws ComponentException {
        this.constrParams = constrParams;
        this.implementation = implementation;
        checkImplementation(implementation);
        setComponentName(name);
        constructor = getContructor();
    }

    protected ConstrComponentFactory(Component comp) {
        this.constrParams = Dependency.EMPTY_PARAMETERS;
        this.implementation = comp.getClass();
        setComponentName(ComponentName.newInstance(comp.getClass()));
        constructor = null;
    }

    /**
     * ʵ
     * <br>
     * ΪnullôϵͳĬϸֻһȡù֮
     * ݹԶȡöӦ
     * <pre>
     *      registry.registerComponent(TestComponent.class)
     *
     *      or
     *
     *      registry.registerComponent(TestComponent.class,null)
     * </pre>
     * <br>
     * Ϊempty (new Paramter[0])ϵͳʹûвĹ
     * <pre>
     *      registry.registerComponent(TestComponent.class,new Parameter[0])
     *
     *      or
     *
     *      registry.registerComponent(TestComponent.class,new Parameter[0],null)
     * </pre>
     *
     * @return
     * @throws org.jfox.ioc.exception.ComponentException
     */
    public Component makeComponent() throws ComponentException {
        if(instantiating == true) {
            throw new ComponentException("cycle depencies in component " + getImplementation().getName());
        }
        instantiating = true;

        try {
            Dependency[] params = getParameters();
            Object[] paramValues = new Object[]{};
            if(params != null) {
                paramValues = resolveParameters(params);
            }
            Component comp = instantiateComponent(constructor, paramValues);

            if(comp instanceof InstantiatedComponent) {
                comp = ((InstantiatedComponent) comp).afterInstantiate(comp);
            }

            return comp;
        }
        finally {
            instantiating = false;
        }

    }

    /**
     * õע
     */
    public ComponentName getComponentName() {
        return compName;
    }

    /**
     * ע
     */
    protected void setComponentName(ComponentName compName) {
        this.compName = compName;
    }

    /**
     * õ
     */
    public Class getImplementation() {
        return implementation;
    }

    /**
     * õעRegistry
     * @return
     */
    public Registry getRegistry() {
        return registry;
    }

    /**
     * עRegistry
     * @param registry
     */
    public void setRegistry(Registry registry) {
        this.registry = registry;
    }

    /**
     * õĲ
     * @return
     */
    public Dependency[] getParameters() {
        return constrParams;
    }

    private Constructor getContructor() throws ComponentException {
        // not specified, must have only 1 constructor
        // then, auto build the ParamPack
        Class[] types = getParameterTypes(constrParams);

        Constructor constructor = null;
        try {
            constructor = Classes.getConstructor(implementation, types);
        }
        catch(NoSuchMethodException e) {
            throw new ComponentException(e);
        }
        return constructor;
    }

    /**
     * ʵ־幹ʵĹ
     * @param constructor
     * @param params
     * @return
     * @throws ComponentException
     */
    protected Component instantiateComponent(Constructor constructor, Object[] params) throws ComponentException {
        try {
            Object comp = constructor.newInstance(params);
            return (Component) comp;
        }
        catch(Exception e) {
            throw new ComponentException("instantiate component error " + getImplementation().getName(), e);
        }
    }

    /**
     *  implementation Ч
     *
     * @param implementation
     * @throws org.jfox.ioc.exception.ComponentException
     */
    private static void checkImplementation(Class implementation) throws ComponentException {
        if(!Component.class.isAssignableFrom(implementation)) {
            throw new ComponentException("component implementation " + implementation.getName() + " is not a valid component, because it not implements " + Component.class.getName());
        }

        // Assert that the component class is concrete.
        boolean isAbstract = (implementation.getModifiers() & Modifier.ABSTRACT) == Modifier.ABSTRACT;
        boolean isPrivate = (implementation.getModifiers() & Modifier.PRIVATE) == Modifier.PRIVATE;
        boolean isProtected = (implementation.getModifiers() & Modifier.PROTECTED) == Modifier.PROTECTED;
        if(implementation.isInterface() || isAbstract) {
            throw new ComponentException("component implementation is not a concrete class " + implementation.getName());
        }
        if(isPrivate || isProtected) {
            throw new ComponentException("component implementation is a private or protected class " + implementation.getName());
        }
    }

}

