/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */

package org.jfox.ioc.factory;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jfox.ioc.Component;
import org.jfox.ioc.ComponentName;
import org.jfox.ioc.annotation.Interceptable;
import org.jfox.ioc.depend.Dependency;
import org.jfox.ioc.exception.ComponentException;
import org.jfox.ioc.ext.InterceptableComponent;
import org.jfox.ioc.util.Annotations;
import org.jfox.ioc.util.Classes;

/**
 * Ҫʹ{@link org.jfox.ioc.ext.InterceptableComponent}ܣ
 * ʹøComponentFactoryʹʵʱ򣬷صǸʵĶ̬
 *
 * @see  org.jfox.ioc.ext.InterceptableComponent
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public class DynProxyComponentFactory extends ConstrComponentFactory {

    public DynProxyComponentFactory(ComponentName name, Class implementation, Dependency[] params) throws ComponentException {
        super(name,implementation,params);
    }

    /**
     * ʵʱ򣬷صǸʵĶ̬
     * صĴͳʵֵеĽӿ
     *
     * @return ʵĶ̬
     * @throws org.jfox.ioc.exception.ComponentException
     */
    public Component makeComponent() throws ComponentException {
        Component comp = super.makeComponent();
        return (Component) Proxy.newProxyInstance(getImplementation().getClassLoader(),
                getInterfaces(comp.getClass()),
                new DPInvocationHandler(comp));
    }

    /**
     * InvocationHandlerִİ취ǰ {@link org.jfox.ioc.ext.InterceptableComponent}
     *  {@link org.jfox.ioc.ext.InterceptableComponent#preInvoke(java.lang.reflect.Method, Object[])}
     *  {@link org.jfox.ioc.ext.InterceptableComponent#postInvoke(java.lang.reflect.Method, Object)}
     * 
     */
    private class DPInvocationHandler implements InvocationHandler {
        private Component comp = null;
        private boolean isInterceptable = false;

        private Map<Method,Boolean> methods = new HashMap<Method, Boolean>();

        public DPInvocationHandler(Component comp) {
            this.comp = comp;
            if(comp instanceof InterceptableComponent) {
                isInterceptable = true;
            }
        }

        public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
            boolean isMethodInterceptable = isMethodInterceptable(method);
            if(isInterceptable && isMethodInterceptable) {
                // ÷עΪһ interceptable ķ
                if(Annotations.isAnnotated(method,Interceptable.class)) {
                    ((InterceptableComponent) comp).preInvoke(method, args);
                }
            }
            Object result = method.invoke(comp, args);
            if(isInterceptable && isMethodInterceptable) {
                // ÷עΪһ interceptable ķ
                if(Annotations.isAnnotated(method, Interceptable.class)) {
                    result = ((InterceptableComponent) comp).postInvoke(method, result);
                }
            }
            return result;
        }

        private boolean isMethodInterceptable(Method method) throws Exception {
            if(methods.containsKey(method)) {
                return methods.get(method);
            }
            else {
                // ෽ûб @Interceptable
                Method concreteMethod = comp.getClass().getMethod(method.getName(),method.getParameterTypes());
                boolean b = Annotations.isAnnotated(concreteMethod,Interceptable.class);
                methods.put(method,b);
                return b;
            }
        }
    }

    /**
     * õʵֵеĽӿڣгʵֵĽӿ
     *
     * @param clazz clazzǽӿ,Ҳʵ
     */
    private static Class[] getInterfaces(Class clazz) {
        List interfaces = Classes.getAllInterfaces(clazz);
        return (Class[])interfaces.toArray(new Class[interfaces.size()]);
    }

}

