/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */

package org.jfox.ioc;

import junit.framework.TestCase;
import org.jfox.ioc.component.TestComponent;
import org.jfox.ioc.component.TestComponentA;
import org.jfox.ioc.component.TestComponentB;
import org.jfox.ioc.component.TestComponentC;
import org.jfox.ioc.component.TestComponentD;
import org.jfox.ioc.depend.ComponentRefDependency;
import org.jfox.ioc.depend.Dependency;
import org.jfox.ioc.depend.DependencyPack;
import org.jfox.ioc.depend.ObjectDependency;

/**
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public class RegistryTest extends TestCase {
    Registry registry;

    public void setUp() throws Exception {
        super.setUp();
        registry = Registry.getInstance();
    }

    public void tearDown() throws Exception {
        super.tearDown();
//        registry.unregisterComponent(ComponentName.newInstance(TestComponentB.class));
    }

    public void testRegisterComponent() throws Exception {
        //ʹ TestComponentA  hashCode Ϊע
        registry.registerComponent(TestComponentA.class);
        TestComponentA ta = (TestComponentA)registry.getComponentInstance(TestComponentA.class);
        assertEquals(ta.getName(),"");
    }

    public void testRegisterComponentByComponentName() throws Exception {
        ComponentName name = ComponentName.newInstance(TestComponentA.class,"test");
        registry.registerComponent(name,name.getType());
        TestComponentA ta = (TestComponentA) registry.getComponentInstance(name);
        assertEquals(ta.getName(), "");
        ta.setName("Hello,World!");
        TestComponentA taa = (TestComponentA) registry.getComponentInstance(name);
        assertEquals(taa.getName(), "Hello,World!");
    }

    public void testRegisterComponentNoneParameterConstructor() throws Exception {
        ComponentMeta meta = registry.registerComponent(TestComponentB.class,
                Dependency.EMPTY_PARAMETERS);
        System.out.println("ComponentMeta: " + meta);
        System.out.println("ComponentName: " + meta.getComponentName());

        TestComponent test = (TestComponent) registry.getComponentInstance(meta.getComponentName());

        System.out.println("ComponentMeta: " + meta);
        assertEquals(test.getName(), "Hello,World!");
    }

    public void testRegisterComponentWithParameterConstructor() throws Exception {
        ComponentName name = ComponentName.newInstance(TestComponentB.class,"parameter");
        ComponentMeta meta = registry.registerComponent(name,
                                                        name.getType(),
                                                        new Dependency[]{
                                                            new ObjectDependency("Hello,World!")
                                                        });
        System.out.println("ComponentMeta: " + meta);
        System.out.println("ComponentName: " + meta.getComponentName());

        TestComponent test = (TestComponent) registry.getComponentInstance(meta.getComponentName());

        System.out.println("ComponentMeta: " + meta);
        assertEquals(test.getName(), "Hello,World!");
    }

    public void testRegisterComponentWithSetter() throws Exception {
        ComponentMeta meta = registry.registerComponent(TestComponentC.class,
                                                        Dependency.EMPTY_PARAMETERS,
                                                        new DependencyPack[]{
                                                            new DependencyPack("name", new ObjectDependency("Hello,Everybody!"))
                                                        },
                                                        true,
                                                        false);
        System.out.println("ComponentMeta: " + meta);
        System.out.println("ComponentName: " + meta.getComponentName());

        TestComponentC test = (TestComponentC) registry.getComponentInstance(meta.getComponentName());

        System.out.println("ComponentMeta: " + meta);
        assertEquals(test.getName(), "Hello,Everybody!");
    }


    public void testRegistryMulti() throws Exception {
        ComponentName name1 = ComponentName.newInstance(TestComponentA.class, "test1");
        registry.registerComponent(name1,name1.getType());
        ComponentName name2 = ComponentName.newInstance(TestComponentA.class, "test2");
        registry.registerComponent(name2,name1.getType());

        TestComponentA ta1 = (TestComponentA)registry.getComponentInstance(name1);
        ta1.setName("Hello,World!");
        TestComponentA ta2 = (TestComponentA) registry.getComponentInstance(name2);
        ta2.setName("Hello,Everybody!");

        assertTrue(registry.countComponents()>=2);
        assertTrue(registry.countComponentTypes() >= 1);

        TestComponentA ta3 = (TestComponentA) registry.getComponentInstance(name1);
        assertEquals(ta3.getName(), "Hello,World!");

        TestComponentA ta4 = (TestComponentA) registry.getComponentInstance(name2);
        assertEquals(ta4.getName(), "Hello,Everybody!");

    }


    public void testConstructorComponentInjection() throws Exception {
        registry.registerComponent(TestComponentD.class,new Dependency[]{
            new ComponentRefDependency(ComponentName.newInstance(TestComponentC.class))
        });

        TestComponentD td = (TestComponentD)registry.getComponentInstance(TestComponentD.class);

        TestComponentC tc = td.getTestComponent();
        assertEquals(tc.getName(), "Hello,Everybody!");

    }

    public void testRegisterByInterface() throws Exception {
        registry.registerComponent(ComponentName.newInstance(Component.class,"TestA"),
                                   TestComponentA.class,
                                   new Dependency[]{},
                                   new DependencyPack[]{
                                    new DependencyPack("name",new ObjectDependency("Hello!"))
                                   },
                                   true,
                                   false);

        TestComponentA ta = (TestComponentA)registry.getComponentInstance(ComponentName.parseString(Component.class.getName() + "@TestA"));
        assertEquals(ta.getName(),"Hello!");
    }

}