/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */
package org.jfox.deployment;

import java.io.File;
import java.io.FileFilter;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.jfox.deployment.application.EARDeployer;
import org.jfox.deployment.application.EARModule;
import org.jfox.deployment.ejb.EJBDeployer;
import org.jfox.deployment.ejb.EJBModule;
import org.jfox.deployment.web.WebDeployer;
import org.jfox.deployment.web.WebModule;
import org.jfox.ioc.ComponentContext;
import org.jfox.ioc.common.AbstractService;
import org.jfox.ioc.deploy.DeployException;
import org.jfox.ioc.deployment.Deployable;
import org.jfox.ioc.deployment.Deployer;
import org.jfox.ioc.ext.ActiveComponent;
import org.jfox.ioc.util.FileFilters;
import org.jfox.ioc.util.Files;

/**
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public class J2EEDeployMonitor extends AbstractService implements
        ActiveComponent {
    private long sleepTime = 1000L;

    private EARDeployer earDeployer;

    private EJBDeployer ejbDeployer;

    private WebDeployer webDeployer;

    private String moniteDirectory = System.getProperty("JFOXCO_HOME", ".")
            + File.separator + "deploy";

    private static String deploymentDir = System
            .getProperty("JFOXCO_HOME", ".")
            + File.separator + "temp";

    // file => last modify
    private Map deployed = new HashMap();

    // file => modules
    private Map deployedModules = new HashMap();

    private static FileFilter filter = FileFilters
            .suffixFileFilter(new String[] { ".jar", ".war", ".ear" });

    /**
     * ͨⲿ÷ʱͣԶ
     */
    private boolean pause = false;

    public J2EEDeployMonitor(EARDeployer earDeployer) {
        this.earDeployer = earDeployer;
        this.ejbDeployer = earDeployer.getEjbDeployer();
        this.webDeployer = earDeployer.getWebDeployer();
    }

    public void setSleepTime(long time) {
        this.sleepTime = time;
    }

    public long getSleepTime() {
        return sleepTime;
    }

    public File[] getDeployedJars() {
        return (File[]) deployed.values().toArray(new File[deployed.size()]);
    }

    public void run() {
        File monitorDir = new File(moniteDirectory);
        while (isStarted()) {
            if (!pause && monitorDir.exists() && monitorDir.isDirectory()) {

                Map newJars = new HashMap();
                // ȡĿ¼е .jar ļ
                File[] jarFiles = monitorDir.listFiles(filter);
                for (int j = 0; j < jarFiles.length; j++) {
                    File jar = jarFiles[j];
                    try {
                        // TODO: ﵥΪһ deploy(File jar) Ըⲿ
                        newJars.put(jar, new Long(jar.lastModified()));
                        _deploy(jar);
                    }
                    catch (Exception e) {
                        logger.warn(e.getMessage(), e);
                    }
                }

                // jars ʣµѾɾjar, undeploy 
                for (Iterator it = deployed.keySet().iterator(); it.hasNext();) {
                    File file = (File) it.next();
                    Deployable module = (Deployable) deployedModules.get(file);
                    Deployer deployer = getDeployer(file);
                    try {
                        deployer.undeploy(module);
                        deployedModules.remove(file);
                    }
                    catch (DeployException e) {
                        logger.warn("undeploy module " + module + " error", e);
                    }
                }
                deployed = newJars;
            }
            try {
                Thread.sleep(sleepTime);
            }
            catch (InterruptedException e) {
                // e.printStackTrace();
                logger.warn(e.getMessage(), e);
            }
        }

    }

    /**
     * ⲿø÷ JAR WAR EAR
     * ļĿ¼Ӧò deploy Ŀ¼£ʱԶļ־ deploy Ŀ¼
     * @param jar
     * @throws DeployException
     */
    public void deploy(File jar) throws DeployException {
        pause = true;
        try {
            for (Iterator it = deployed.keySet().iterator(); it.hasNext();) {
                File deployedJar = (File) it.next();
                if (deployedJar.getName().equalsIgnoreCase(jar.getName())) {
                    String msg = "File "
                            + jar.getName()
                            + " has been deployed, please undeploy it first or rename the file and redeploy.";
                    logger.warn(msg);
                    throw new DeployException(msg);
                }
            }
            
            //Ҫļ deploy Ŀ¼
            try {
                Files.copy(jar, new File(moniteDirectory));
            }
            catch(Exception e){
                throw new DeployException("copy jar " + jar + " to deploy directory error", e);
            }
            File newJar = new File(moniteDirectory,jar.getName()); 
            _deploy(newJar);

        }
        finally {
            pause = false;
        }
    }

    /**
     * ԶĵjarļķҪжϸjarǷѾ
     * 
     * @param jar
     */
    private void _deploy(File jar) throws DeployException {
        if (isToBeDeploy(jar)) { // Ҫ
            Deployer deployer = getDeployer(jar);
            // ˵ģѾǸ£Ӧ undeploy
            if (deployed.containsKey(jar)) {
                Deployable module = (Deployable) deployedModules.get(jar);
                deployer.undeploy(module);
            }

            logger.info("deploy module " + jar.getName() + " ...");
            Deployable module = buildModule(jar);
            deployer.deploy(module);
            deployedModules.put(jar, module);
            logger.info("deploy module " + jar.getName() + " over.");
        }
        deployed.remove(jar);

    }

    protected void doStart() throws Exception {
        Thread deployMonitor = new Thread(this, this.getClass().getName());
        deployMonitor.setPriority(Thread.MIN_PRIORITY);
        deployMonitor.start();
    }

    protected void doStop() throws Exception {

    }

    private boolean isToBeDeploy(File jar) {
        if (!deployed.containsKey(jar)) { //  jar
            return true;
        }
        else {
            long lastModify = ((Long) deployed.get(jar)).longValue();
            if (jar.lastModified() > lastModify) { // ޸ĵ jar
                return true;
            }
            else {
                return false;
            }
        }
    }

    protected void doInit() throws Exception {

    }

    protected void doDestroy() throws Exception {

    }

    public void setComponentContext(ComponentContext ctx) {
        super.setComponentContext(ctx);
        moniteDirectory = ctx.getModuleDir() + File.separator + "apps";
        deploymentDir = ctx.getModuleDir() + File.separator + "temp";
    }

    public static String getDeploymentDir() {
        return deploymentDir;
    }

    private Deployer getDeployer(File jar) {
        if (jar.getName().endsWith(".ear")) {
            return earDeployer;
        }
        else if (jar.getName().endsWith(".war")) {
            return webDeployer;
        }
        else if (jar.getName().endsWith(".jar")) {
            return ejbDeployer;
        }
        return null;
    }

    private Deployable buildModule(File jar) throws DeployException {
        if (jar.getName().endsWith(".ear")) {
            return new EARModule(jar);
        }
        else if (jar.getName().endsWith(".war")) {
            return new WebModule(jar);
        }
        else if (jar.getName().endsWith(".jar")) {
            return new EJBModule(jar);
        }
        return null;

    }

    public static void main(String[] args) {

    }
}