/*
 * Copyright (c) 2004 Your Corporation. All Rights Reserved.
 */
package org.jfox.deployment.web;

import org.jfox.ioc.deploy.DeployException;
import org.jfox.jetty.JettyWebContainer;
import org.mortbay.http.HttpContext;
import org.mortbay.j2ee.J2EEWebApplicationContext;
import org.mortbay.j2ee.session.Manager;

/**
 * @author <a href="mailto:yy.young@gmail.com">Young Yang</a>
 */

public class JettyWebDeployer extends WebDeployer {

    private JettyWebContainer jetty;

    public JettyWebDeployer(JettyWebContainer jetty) {
        this.jetty = jetty;
    }

    protected void doDeploy(WebModule webModule) throws DeployException {
        try {
            J2EEWebApplicationContext webApp = new JettyWebApplicationContext(webModule.getTempDeployDirectory().getCanonicalPath());
            webApp.setTempDirectory(webModule.getTempDeployDirectory());
            if(webModule.getWebContext().equals(jetty.getRootWebApp())) {
                webApp.setContextPath("/");
            }
            else {
                webApp.setContextPath("/" + webModule.getWebContext());
            }
            // set ClassLoader
//      webApp.setClassLoader(webModule.getClassLoader());
            webApp.setParentClassLoader(webModule.getClassLoader());
            webApp.setClassLoaderJava2Compliant(true);

            Manager manager = jetty.getDistributableSessionManagerPrototype();
                        if(manager != null) {
                            webApp.setDistributableSessionManager((Manager) manager.clone());
                            if(jetty.getForceDistributable()) webApp.setDistributable(true);
                        }
            jetty.addContext(webApp);
            webApp.start();
        }
        catch(Exception e) {
            throw new DeployException("deploy web module " + webModule.getWebContext() + " failed", e);
        }

    }

    protected void doUndeploy(WebModule webModule) {
        HttpContext httpContext = jetty.getContext(webModule.getWebContext());
        try {
            httpContext.stop(true);
        }
        catch(Exception e) {
            e.printStackTrace();
        }
        jetty.removeContext(httpContext);
    }

    protected boolean isStarted() {
        return jetty.isStarted();
    }

    protected void doInit() throws Exception {
        if(!jetty.isStarted()) {
            jetty.start();
        }
    }

    protected void doDestroy() throws Exception {

    }

    public static void main(String[] args) {

    }
}

