/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */

package org.jfox.ejb;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.rmi.RemoteException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.ejb.EJBException;
import javax.ejb.EJBHome;
import javax.ejb.EJBMetaData;
import javax.ejb.EnterpriseBean;
import javax.ejb.HomeHandle;
import javax.ejb.TimedObject;
import javax.ejb.TimerService;
import javax.naming.LinkRef;
import javax.naming.NamingException;

import org.jfox.ejb.connector.EJBConnectorInvoker;
import org.jfox.ejb.connector.EJBInvocation;
import org.jfox.ejb.handler.HandlerChain;
import org.jfox.ejb.meta.EJBDescriptor;
import org.jfox.ejb.meta.EJBReferenceDescriptor;
import org.jfox.ejb.meta.EnvEntryDescriptor;
import org.jfox.ejb.meta.ResourceEnvRefDescriptor;
import org.jfox.ejb.meta.ResourceRefDescriptor;
import org.jfox.ejb.timer.EJBTimerService;
import org.jfox.ejb.timer.TimedObjectMethod;
import org.jfox.ioc.common.AbstractComponent;
import org.jfox.ioc.util.Methods;
import org.jfox.jndi.InitialContextHelper;
import org.jfox.jndi.enc.EnterpriseContext;

/**
 * ṩ bucket Ĺ
 *
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public abstract class BucketSupport extends AbstractComponent implements Bucket {
//  protected Logger logger = Logger.getLogger(getClass().getName());
    protected BucketMeta bucketMeta = null;
//  protected Container container = null;

    // homeClassName ΪObjectId ǰ׺
    protected String homeClassName = null;
    /**
     * EJBObject ӿжķ
     * methodHashCode => Method Object
     */
    protected Map beanMethods = new HashMap();
    protected Map homeMethods = new HashMap();

    protected static Map ejbObjectMethods = new HashMap(5);
    static {
        ejbObjectMethods.put(String.valueOf(Methods.getMethodHash(EJBObjectMethod.GetEJBHome)),EJBObjectMethod.GetEJBHome);
        ejbObjectMethods.put(String.valueOf(Methods.getMethodHash(EJBObjectMethod.GetHandle)), EJBObjectMethod.GetHandle);
        ejbObjectMethods.put(String.valueOf(Methods.getMethodHash(EJBObjectMethod.GetPrimaryKey)), EJBObjectMethod.GetPrimaryKey);
        ejbObjectMethods.put(String.valueOf(Methods.getMethodHash(EJBObjectMethod.IsIdentical)), EJBObjectMethod.IsIdentical);
        ejbObjectMethods.put(String.valueOf(Methods.getMethodHash(EJBObjectMethod.Remove)), EJBObjectMethod.Remove);
    }

    protected EJBObjectIDGenerator idgen = null;

//  protected List interceptors = new ArrayList();
    protected HandlerChain handlerChain = null;

    // enc
    protected EnterpriseContext enc = new EnterpriseContext();

    protected TimerService timerService;

    public BucketSupport() {

    }

    public void setBucketMeta(BucketMeta meta) {
        this.bucketMeta = meta;
    }

    public void setHandlerChain(HandlerChain handlerChain) {
        this.handlerChain = handlerChain;
    }

    public ClassLoader getEJBClassLoader() {
        return bucketMeta.getBeanClass().getClassLoader();
    }

    public EJBMetaData getEJBMetaData() throws RemoteException {
        return bucketMeta.getEJBMetaData();
    }

    public EJBHome getEJBHome() throws RemoteException {
        return bucketMeta.getEJBMetaData().getEJBHome();
    }

    public HomeHandle getHomeHandle() throws RemoteException {
        return new HomeHandleImpl(bucketMeta.getEJBDescriptor().getJndiName());
    }

    public EnterpriseContext getEnterpriseContext() {
        return enc;
    }

    // call back the ejbCreate method
    public void ejbCreate(EnterpriseBean bean, String createMethod, Object[] args) throws RemoteException {
        char c = createMethod.charAt(0);
        String ejbCreateMethodName = "ejb" + Character.toUpperCase(c) + createMethod.substring(1);

        logger.debug(bean + " " + ejbCreateMethodName);
        Class[] signatures = null;
        if(args != null && args.length != 0) {
            signatures = new Class[args.length];
            for(int i = 0; i < args.length; i++) {
                signatures[i] = args[i].getClass();
            }
        }

        try {
            bean.getClass().getMethod(ejbCreateMethodName, signatures).invoke(bean, args);
        }
        catch(Exception e) {
            throw new RemoteException(e.getMessage(), e);
        }
    }

    /**
     *  EJBHome ʹ DelegateInvocationHandler, ɿͻʹʲôЭ
     *
     * @return
     */
    private EJBHome createEJBHome() {
        EJBHome home = (EJBHome) Proxy.newProxyInstance(bucketMeta.getEJBMetaData().getHomeInterfaceClass().getClassLoader(),
                new Class[]{bucketMeta.getEJBMetaData().getHomeInterfaceClass()},
                new EJBConnectorInvoker(this.getHomeObjectId(),null));
        logger.debug("createEJBHome " + homeClassName + " Successful");
        return home;
    }

    /**
     *  EJBHome,EJBObject ӿжĶ
     *
     * @param classObj
     * @return
     */
    protected Map introspectorHomeMethods(Class classObj) {
        Map homeMethods = new HashMap();
        Method[] methods = classObj.getMethods();
        for(int i = 0; i < methods.length; i++) {
            long hashCode = Methods.getMethodHash(methods[i]);
            homeMethods.put(String.valueOf(hashCode), methods[i]);
            logger.debug("Cache Home method: " + hashCode + " => " + classObj.getName() + "." + methods[i].getName());
        }
        return homeMethods;
    }

    /**
     *  EnterpriseBean жĶ
     * ЩӦöѾ EJBObject ӿѾ
     *
     * @param beanClass
     * @return
     */
    protected Map introspectorBeanMethods(Class beanClass, Class remoteInterface) {
        Map methods = new HashMap();
        Method[] remoteMethods = remoteInterface.getDeclaredMethods();
        for(int i = 0; i < remoteMethods.length; i++) {
            Method _method = remoteMethods[i];
            try {
                Method method = beanClass.getMethod(_method.getName(), _method.getParameterTypes());
                long hashCode = Methods.getMethodHash(method);
                methods.put(String.valueOf(hashCode), method);
                logger.debug("Cache Bean method: " + hashCode + " => " + beanClass.getName() + "." + method.getName());
            }
            catch(Exception e) {
                throw new EJBException(e);
            }
        }

        // TimedObject
        if(TimedObject.class.isAssignableFrom(beanClass)) {
            long hashCode = Methods.getMethodHash(TimedObjectMethod.TimeOut);
            methods.put(String.valueOf(hashCode), TimedObjectMethod.TimeOut);
            logger.debug("Cache business method: " + hashCode + " => " + TimedObjectMethod.TimeOut.getName());
        }

        return methods;
    }


    public Method getHomeMethod(EJBInvocation invocation) throws NoSuchMethodException {
        if(!homeMethods.containsKey(invocation.getMethodHash())) {
            throw new NoSuchMethodException("no such EJBHome method with method hash is " + invocation.getMethodHash());
        }
        return (Method) homeMethods.get(invocation.getMethodHash());
    }

    public Method getBeanMethod(EJBInvocation invocation) throws NoSuchMethodException {
        String hash = invocation.getMethodHash();
        if(beanMethods.containsKey(hash)) {
            return (Method)beanMethods.get(hash);
        }
        if(ejbObjectMethods.containsKey(invocation.getMethodHash())){
           return (Method)ejbObjectMethods.get(hash);
        }
        throw new NoSuchMethodException("no such EnterpriseBean method with method hash is " + invocation.getMethodHash());
    }

    public EJBDescriptor getEJBDescriptor() {
        return bucketMeta.getEJBDescriptor();
    }

    public TimerService getTimerService() {
        return timerService;
    }

    public EJBObjectId nextObjectId() {
        return idgen.nextBeanObjectId();
    }

    public EJBObjectId getHomeObjectId() {
        return idgen.getHomeObjectId();
    }

    public Class getBeanClass() {
        return bucketMeta.getBeanClass();
    }

    /**
     *  EJBHome ķ
     *
     * @param invocation
     * @return
     * @throws Exception
     */
    public Object invokeHome(EJBInvocation invocation) throws Exception {
        logger.debug("invokeHome " + invocation.getMethod().getName());
        return handlerChain.invokeHome(this, invocation);
//        return ((HandlerChain) Registry.getInstance().getComponentInstance(ComponentName.parseString(HandlerChain.class.getName() + "@Stateless"))).invokeHome(this, invocation);
    }

    /**
     * ִ Bean ķ
     *
     * @param invocation
     * @return
     * @throws Exception
     */
    public Object invokeBean(EJBInvocation invocation) throws Exception {
        logger.debug("invokeBean " + invocation.getMethod().getName());
        return handlerChain.invokeBean(this, invocation);

    }

    protected void doInit() throws Exception {
//        logger = Logger.getLogger(this.getClass().getName() + "[" + bucketMeta.getBeanClass().getName() + "]");
        idgen = EJBObjectIDGenerator.newInstance(bucketMeta.getEJBDescriptor().getEjbName(),
                                                 bucketMeta.getEJBMetaData().getHomeInterfaceClass().getName(),
                                                 bucketMeta.getEJBMetaData().getRemoteInterfaceClass().getName());

        this.homeClassName = bucketMeta.getEJBMetaData().getHomeInterfaceClass().getName();
        homeMethods = introspectorHomeMethods(bucketMeta.getEJBMetaData().getHomeInterfaceClass());
        beanMethods = introspectorBeanMethods(bucketMeta.getBeanClass(), bucketMeta.getEJBMetaData().getRemoteInterfaceClass());
//        beanMethods.putAll(introspectorInterfMethods(EJBObject.class));

        timerService = new EJBTimerService(bucketMeta.getEJBDescriptor().getEjbName());

        // setEJBHome , Ա doInit ܹ󶨵 jndi
        ((EJBMetaDataImpl) bucketMeta.getEJBMetaData()).setEJBHome(createEJBHome());

        EJBDescriptor ejbDescriptor = bucketMeta.getEJBDescriptor();
        InitialContextHelper.getInitialContext().bind(ejbDescriptor.getJndiName(), bucketMeta.getEJBMetaData().getEJBHome());

        // bind refrence to EnterpriseContext
        bindReference();

    }

    protected void doDestroy() throws Exception {
        InitialContextHelper.getInitialContext().unbind(bucketMeta.getEJBDescriptor().getJndiName());
    }

    protected void bindReference() {
        EJBDescriptor ejbDescriptor = bucketMeta.getEJBDescriptor();
        try {
            List entries = ejbDescriptor.getEnvEntries();
            for(int i = 0; i < entries.size(); i++) {
                EnvEntryDescriptor env = (EnvEntryDescriptor) entries.get(i);
                String name = env.getName();
                Object value = env.getValueObject();
                enc.bind(name, value);
                logger.debug("bind env-entry, name: " + name);
            }

            List ejbRefs = ejbDescriptor.getEjbReferences();
            for(int i = 0; i < ejbRefs.size(); i++) {
                EJBReferenceDescriptor ejbRef = (EJBReferenceDescriptor) ejbRefs.get(i);
//        checkEjbReference(ejbRef);
                LinkRef linkRef = new LinkRef(ejbRef.getRefName());
                enc.bind(ejbRef.getRefName(), linkRef);
                logger.debug("bind ejb-ref, name: " + ejbRef.getRefName());
            }

            List resRefs = ejbDescriptor.getResourceReferences();
            for(int i = 0; i < resRefs.size(); i++) {
                ResourceRefDescriptor resRef = (ResourceRefDescriptor) resRefs.get(i);
//        checkResourceRef(resRef);
                LinkRef linkRef = new LinkRef(resRef.getRefName());
                enc.bind(resRef.getRefName(), linkRef);
                logger.debug("bind resource-ref, name: " + resRef.getRefName());
            }

            List resourceEnvRefs = ejbDescriptor.getResourceEnvReferences();
            for(int i = 0; i < resourceEnvRefs.size(); i++) {
                ResourceEnvRefDescriptor resourceEnvRef = (ResourceEnvRefDescriptor) resourceEnvRefs.get(i);
//        checkResourceEnvRef(resourceEnvRef);
                LinkRef linkRef = new LinkRef(resourceEnvRef.getRefName());
                enc.bind(resourceEnvRef.getRefName(), linkRef);
                logger.debug("bind resource-env-ref, name: " + resourceEnvRef.getRefName());
            }

        }
        catch(NamingException e) {
            logger.warn("bind EnterpriseContext error", e);
        }
    }


}

