/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */

package org.jfox.ejb;

import java.lang.reflect.Proxy;
import java.rmi.RemoteException;
import javax.ejb.EJBContext;
import javax.ejb.EJBException;
import javax.ejb.EnterpriseBean;
import javax.ejb.Handle;
import javax.ejb.MessageDrivenBean;
import javax.ejb.MessageDrivenContext;
import javax.ejb.RemoveException;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.Session;

import org.jfox.ejb.connector.EJBInvocation;
import org.jfox.ejb.meta.MDBDescriptor;
import org.jfox.ejb.timer.EJBTimerService;
import org.jfox.ioc.ComponentName;
import org.jfox.ioc.connector.ConnectorHelper;
import org.jfox.ioc.connector.ConnectorRemote;
import org.jfox.ioc.connector.ObjectId;
import org.jfox.ioc.depend.DependencyPack;
import org.jfox.ioc.depend.ObjectDependency;
import org.jfox.ioc.util.ObjectUUID;
import org.jfox.jms.JMSConnectionRemote;
import org.jfox.jms.JMSConsumer;
import org.jfox.jms.connector.JMSConnectorInvoker;
import org.jfox.jms.connector.JMSContainer;
import org.jfox.jms.message.JMSMessage;
import org.jfox.jndi.InitialContextHelper;
import org.jfox.jndi.JNDIProperties;

/**
 * һMessage Driven Bean СյϢ֮󣬰ϢBeanִ
 *
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public class MDBBucket extends BucketSupport {
    /*
    * ̬MDBBucketһ conn
    */
    private static MDBConnection conn = null;

    private MDBSession session = null;

    private MDBObjectPool instancePool = null;

    private MessageDrivenContext ejbConext = null;

    private String user = null;
    private String password = null;

    public MDBBucket() {
        super();
    }

    //########### EJBObject ķ
    public Object getPrimaryKey(EnterpriseBean obj) throws RemoteException {
        throw new EJBException("NOT SUPPORT!");
    }

    public Handle getHandle(EnterpriseBean obj) throws RemoteException {
        throw new EJBException("NOT SUPPORT!");
    }

    public boolean isIdentical(EJBObjectId thisObjectId, EJBObjectId thatObjectId) throws RemoteException {
        throw new EJBException("NOT SUPPORT!");
    }

    public void remove(EnterpriseBean bean) throws RemoteException, RemoveException {
        ejbRemove(bean);
    }

    public void remove(Handle handle) throws RemoteException, RemoveException {
        throw new EJBException("NOT SUPPORT!");
    }

    public void remove(Object primaryKey) throws RemoteException, RemoveException {
        throw new EJBException("NOT SUPPORT!");
    }

    /**
     * MDB No EJBHome
     * @param invocation
     * @return
     * @throws Exception
     */
    public Object invokeHome(EJBInvocation invocation) throws Exception {
        throw new EJBException("MessageDrivenBean has no EJBHome.");
    }

    public Object invokeBean(EJBInvocation invocation) throws Exception {
        return super.invokeBean(invocation);
    }

    public void ejbActivate(EnterpriseBean bean) throws EJBException, RemoteException {
        throw new EJBException("NOT SUPPORT!");
    }

    public void ejbPassivate(EnterpriseBean bean) throws EJBException, RemoteException {
        throw new EJBException("NOT SUPPORT!");
    }

    public void ejbRemove(EnterpriseBean bean) throws EJBException, RemoteException {
        logger.debug(bean + " remove");
        ((MessageDrivenBean)bean).ejbRemove();
    }

    public void setContext(EnterpriseBean bean, EJBContext ctx) throws EJBException, RemoteException {
        logger.debug(bean + " setMessageDrivenContext");
        ((MessageDrivenBean)bean).setMessageDrivenContext((MessageDrivenContext)ctx);
    }

    public EJBPoolableObject retrieveBean(Object key) throws Exception {
        MDBPoolableObject pobj = (MDBPoolableObject) instancePool.retrieveObject();
        if(!pobj.isEjbCreated()) { // ĶҪ setSessionContext, ejbCreate
            MessageDrivenBean bean = (MessageDrivenBean) pobj.getBeanInstance();
            setContext(bean, ejbConext);
            ejbCreate(bean, "create", null);
            pobj.ejbCreated();
        }
        return pobj;
    }

    public void restoreBean(Object key, EJBPoolableObject pobj) {
        instancePool.restoreObject(pobj);
    }

    public EJBObjectId createBean(EJBInvocation invocation) throws Exception {
        throw new EJBException("NOT Support in MDB!");
    }

	public void syncBean(EJBInvocation invocation) {
		throw new EJBException("NOT Support in MDB!");
	}

	/**
     * Ҫ ConnectionFactory.createConnection 
     * MDBBucket һ Connection
     * @throws Exception
     */
    protected void doInit() throws Exception {
        idgen = EJBObjectIDGenerator.newInstance(bucketMeta.getEJBDescriptor().getEjbName(),
                                                 bucketMeta.getBeanClass().getName(),
                                                 bucketMeta.getBeanClass().getName());

        beanMethods = introspectorBeanMethods(MDBConnection.class,JMSConnectionRemote.class);

        timerService = new EJBTimerService(bucketMeta.getEJBDescriptor().getEjbName());

        // bind refrence to EnterpriseContext
        bindReference();

        // create connection if needed
        if(conn == null) {
            conn = createConnection(user,password,false);
            conn.start();
        }

        // craete Consumer
        instancePool = (MDBObjectPool) context.getRegistry().getComponentInstance(
                                                                ComponentName.newInstance(MDBObjectPool.class),
                                                                new DependencyPack[]{
                                                                    new DependencyPack("EJBObjectFactory",
                                                                                      new ObjectDependency(new MDBObjectFactory(MDBPoolableObject.class, this)))});
        // create session
        session = (MDBSession) conn.createSession(false, Session.AUTO_ACKNOWLEDGE);
        String destinationJndiName = ((MDBDescriptor) bucketMeta.getEJBDescriptor()).getDestination();
        Destination dest = (Destination)InitialContextHelper.getInitialContext().lookup(destinationJndiName);
        // һConsumerܽյõַϢMDBSessionյϢ֮󣬽 pool ȡMDBʵӦ
        JMSConsumer consumer =  (JMSConsumer)session.createConsumer(dest);
        // 첽
        session.setConsumerAsync(consumer,true);

    }

    protected void doDestroy() throws Exception {
        // MDB ҪJNDIж
//        super.doDestroy();
        instancePool.destroy();
    }

    public void onMessage(String sessionId, MessageDrivenBean bean, JMSMessage msg) throws RemoteException, JMSException {
        conn.onMessage(sessionId,bean,msg);
    }

    protected synchronized MDBConnection createConnection(String userName, String password, boolean isXA) throws JMSException {

        ObjectId clientId = new ObjectId(ObjectUUID.randomUUID());
        // lookup ConnectorService from jndi, Ȼʹö̬װ JMSContainer 
        ConnectorRemote remote = ConnectorHelper.lookupConnector(JNDIProperties.getProviderURL());

        JMSContainer container = null;

        if(container == null) {
            container = (JMSContainer) Proxy.newProxyInstance(this.getClass().getClassLoader(),
                                                              new Class[]{JMSContainer.class},
                                                              new JMSConnectorInvoker(clientId, remote));
        }

        container.auth(userName, password);

        MDBConnection conn = new MDBConnection(clientId.toString(), container, isXA);
        try {
            //  MDBConnection Ĵעᵽ JMSContainer
            // Ҫı onMessage Ĳ,ʹ ConnectorRemote
            Object proxy = Proxy.newProxyInstance(this.getClass().getClassLoader(),
                                                  new Class[]{JMSConnectionRemote.class},
                                                  new MDBConnectorInvoker(nextObjectId(),remote));
            container.registerConnection(clientId.toString(), proxy);
        }
        catch(Exception e) {
            e.printStackTrace();
        }
        return conn;
    }

    public static void main(String[] args) {

    }

}
