/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */

package org.jfox.ejb;

import java.lang.reflect.Proxy;
import java.rmi.RemoteException;
import javax.ejb.EJBObject;
import javax.ejb.EnterpriseBean;
import javax.ejb.Handle;
import javax.ejb.RemoveException;
import javax.ejb.SessionBean;

import org.jfox.ejb.connector.EJBInvocation;
import org.jfox.ioc.ComponentName;
import org.jfox.ioc.connector.ConnectorInvoker;
import org.jfox.ioc.depend.DependencyPack;
import org.jfox.ioc.depend.ObjectDependency;

/**
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public class StatefulBucket extends SessionBucket {

    private StatefulObjectPool instancePool = null;

    public StatefulBucket() {

    }

    public void remove(EnterpriseBean bean) throws RemoteException, RemoveException {
        ejbRemove((SessionBean) bean);
    }

    public void remove(Handle handle) throws RemoteException, RemoveException {
        EJBObject ejbObject = handle.getEJBObject();
        // Handle ʵһ󣬱 ObjectId 
        EJBObjectId objectId = (EJBObjectId)(((ConnectorInvoker) Proxy.getInvocationHandler(ejbObject)).getClientId());
        try {
            EJBPoolableObject pobj = (EJBPoolableObject) instancePool.retrieveObject(objectId);
            remove(pobj.getBeanInstance());
        }
        catch(Exception e) {
            throw new RemoveException(e.getMessage());
        }
    }

    public boolean isIdentical(EJBObjectId thisObjectId, EJBObjectId thatObjectId) throws RemoteException {
        return thisObjectId.equals(thatObjectId);
    }

    public EJBPoolableObject retrieveBean(Object key) {
        try {
            return (EJBPoolableObject) instancePool.retrieveObject(key);
        }
        catch(Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    public void restoreBean(Object key, EJBPoolableObject pobj) {
        instancePool.restoreObject(key, pobj);
    }

    public EJBObjectId createBean(EJBInvocation invocation) throws Exception {
        EJBObjectId beanObjectId = nextObjectId();
        EJBPoolableObject pobj = (EJBPoolableObject) instancePool.createObject();
        SessionBean bean = (SessionBean) pobj.getBeanInstance();
        setContext(bean, new SessionContextImpl(beanObjectId, bucketMeta));
        ejbCreate(bean, invocation.getMethod().getName(), invocation.getArgs());
        instancePool.restoreObject(beanObjectId, pobj);
        return beanObjectId;
    }

	public void syncBean(EJBInvocation invocation) {
		ClassLoader oldCL = Thread.currentThread().getContextClassLoader();
		ClassLoader cl = this.getEJBClassLoader();
		try {
			Thread.currentThread().setContextClassLoader(cl);
			//invocation.getTargetObject ҪõEJBClassLoader
			EnterpriseBean bean = (EnterpriseBean) invocation.getTargetObject();
			EJBPoolableObject pobj = (EJBPoolableObject) instancePool.createObject();
			pobj.setBeanInstance(bean);
			//TODO: ejbObjectIdѾˣҪ
			this.restoreBean(invocation.getObjectId(), pobj);
		}
		catch(Exception e) {
			logger.warn("pool.createObject failed.", e);
		}
		finally {
			Thread.currentThread().setContextClassLoader(oldCL);
		}
	}

	protected void doInit() throws Exception {
//        instancePool = new StatefulObjectPool(this, new EJBObjectFactory(EJBPoolableObject.class, bucketMeta.getBeanClass()));
//        instancePool.init();
        super.doInit();
        instancePool = (StatefulObjectPool) context.getRegistry().getComponentInstance(
                ComponentName.newInstance(StatefulObjectPool.class),
                new DependencyPack[]{new DependencyPack("EJBObjectFactory",
                                                      new ObjectDependency(new StatefulObjectFactory(StatefulPoolableObject.class, this))),
                                    });

    }

    protected void doDestroy() throws Exception {
        super.doDestroy();
        //  instancePool ʱ򣬻леʵҲ LifeCycleManager ļ
        instancePool.destroy();
    }

    public static void main(String[] args) {

    }
}