/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */

package org.jfox.ejb;

import java.rmi.NoSuchObjectException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import javax.ejb.SessionBean;

import org.jfox.ioc.ComponentName;
import org.jfox.ioc.common.AbstractComponent;
import org.jfox.persistent.Persistenter;
import org.jfox.pool.AbstractObjectPool;
import org.jfox.pool.IndexableObjectPool;
import org.jfox.pool.ObjectFactory;
import org.jfox.pool.PoolableObject;
import org.jfox.pool.SimpleObjectPool;

/**
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public class StatefulObjectPool extends AbstractComponent implements IndexableObjectPool,EJBObjectPool {
    private AbstractObjectPool pool = null;
    private EJBObjectFactory factory = null;
    // ʹõĶ(ObjectID <=> EJB)
    private Map caches = null;
    // ¼ÿ Bean 󷵻 pool ʱ
    private Map timeouts = null;

    private Bucket bucket = null;

    // bean ʱ
    final static long MAX_LIFE_TIME = 60 * 60 * 1000L; // one hour
    private final static long MAX_PASSIVATE_TIME = MAX_LIFE_TIME * 24 * 7; // one week

    private static StatefulManager lifecycleManger = new StatefulManager();
    private Persistenter persistent = null;

    public StatefulObjectPool() {

    }

    public void setEJBObjectFactory(EJBObjectFactory factory) {
        this.factory = factory;
    }

    public PoolableObject createObject() throws Exception {
        return pool.retrieveObject();
    }

    public synchronized PoolableObject retrieveObject(Object key) throws Exception {
        //  bean ˣԴ timeout ɾʱ restore ʱ¿ʼʱ
        timeouts.remove(key);
        // ڶ cache УпѾ passivate 
        if(!caches.containsKey(key)) {
            try {
                EJBPoolableObject pobj = activateObject(key);
                //Ҫŵcaches ,ڵɺ, restoreObjectʱٷ
//        caches.put(key,pobj);
                return pobj;
            }
            catch(Exception e) {
                e.printStackTrace();
            }
            //  passivate Ҳ׳쳣
            throw new NoSuchObjectException(key.toString());
        }
        else {
            return (PoolableObject) caches.remove(key);
        }
    }

    /**
     * Ѿи key ڣ falseû򷵻 true
     *
     * @param key
     * @param obj
     * @return
     */
    public boolean restoreObject(Object key, PoolableObject obj) {
        // ¼󷵻صʱ䣬ʱ䳬 60mл
        timeouts.put(key, new Long(System.currentTimeMillis()));

        if(caches.containsKey(key)) {
            logger.warn("key " + key + " is existed,overwrite!");
            caches.put(key, obj);
            return false;
        }
        else {
            caches.put(key, obj);
            return true;
        }
    }

    public boolean removeObject(Object key) {
        logger.debug("removeObject with key - " + key);
        timeouts.remove(key);
        try {
            SessionBean sb = loadObject(key);
            if(sb != null) {
                bucket.ejbRemove(sb);
                persistent.remove(key);
                return true;
            }
            else {
                return false;
            }
        }
        catch(Exception e) {
            logger.warn(e);
            return false;
        }
    }

    public boolean passivateObject(Object key) {
        if(!caches.containsKey(key)) return false;
        logger.debug("passivateObject with key - " + key);
        // Ҫ timeouts Ƴ,ΪҪΪ removeObject ʱ
//    timeouts.remove(key);
        EJBPoolableObject pobj = (EJBPoolableObject) caches.remove(key);
        try {
            bucket.ejbPassivate((SessionBean) pobj.getBeanInstance());
            persistent.store(key, pobj.getBeanInstance());
            pobj = null;
            return true;
        }
        catch(Exception e) {
            logger.warn(e);
            return false;
        }
    }

    public EJBPoolableObject activateObject(Object key) throws Exception {
        logger.debug("activateObject with key - " + key);
        SessionBean sb = loadObject(key);
        bucket.ejbActivate(sb);
        EJBPoolableObject pobj = new EJBPoolableObject();
        pobj.setBeanInstance(sb);
        return pobj;
    }

    private SessionBean loadObject(Object key) throws Exception {
//    logger.debug("loadObject with key - " + key );
        SessionBean sb = (SessionBean) persistent.load(key);
        return sb;
    }

    public void clear() {
        synchronized(caches) {
            for(Iterator it = caches.values().iterator(); it.hasNext();) {
                EJBPoolableObject pobj = (EJBPoolableObject) it.next();
                try {
                    bucket.ejbPassivate((SessionBean) (((EJBPoolableObject) pobj).getBeanInstance()));
                }
                catch(Exception e) {
                    logger.warn("ejbRemove failed, exception is: " + e.toString());
                }
            }
            caches.clear();
        }
        timeouts.clear();
        pool.clear();
    }

    public ObjectFactory getObjectFactory() {
        return factory;
    }

    public int getWorking() {
        return caches.size();
    }

    public int getRest() {
        return pool.getRest();
    }

    protected void doInit() throws Exception {
        if(factory == null) throw new NullPointerException("factory is null.");
        bucket = factory.getBucket();
        timeouts = new HashMap();
        caches = new HashMap();
        pool = new SimpleObjectPool(factory);
        pool.init();

        persistent = (Persistenter)context.getRegistry().getComponentInstance(ComponentName.parseString("org.jfox.persistent.Persistenter@File"));

        lifecycleManger.manageIt(this);
    }

    protected void doDestroy() throws Exception {
        clear();
        lifecycleManger.unmanageIt(this);
        caches = null;
        pool = null;
    }

    /**
     * ԻĶڼ
     */
    public void lifecycleCheck() {
        logger.debug("Lifecycle checking... " + timeouts.size() + ", " + bucket.getEJBDescriptor().getRemoteClassName());
        long now = System.currentTimeMillis();
//    logger.info("sleep time " + now + ", " + lastActive);
        if(timeouts.size() == 0) return;
        for(Iterator it = new HashMap(timeouts).entrySet().iterator(); it.hasNext();) {
            Map.Entry entry = (Map.Entry) it.next();
            long lastActive = ((Long) entry.getValue()).longValue();

            if((now - lastActive) > MAX_PASSIVATE_TIME) {
                removeObject(entry.getKey());
                continue;
            }
            if((now - lastActive) > MAX_LIFE_TIME) {
                passivateObject(entry.getKey());
            }
        }
    }

    public static void main(String[] args) {

    }
}