/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */

package org.jfox.ejb;

import java.rmi.RemoteException;
import javax.ejb.EnterpriseBean;
import javax.ejb.Handle;
import javax.ejb.RemoveException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.jfox.ejb.connector.EJBInvocation;
import org.jfox.ioc.ComponentName;
import org.jfox.ioc.depend.DependencyPack;
import org.jfox.ioc.depend.ObjectDependency;

/**
 * ִ EJBObject EJBHome жķ
 *
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public class StatelessBucket extends SessionBucket {

    private StatelessObjectPool instancePool = null;
    /**
     * StatlessSessionBean ÿηͬObjectId
     */
    private EJBObjectId beanObjectId = null;
    private SessionContext ejbConext = null;

    public StatelessBucket() {

    }

    //  StatelessSessionBeanִκβ
    public void remove(Handle handle) throws RemoteException, RemoveException {
        try {
            EJBPoolableObject pobj = (EJBPoolableObject) instancePool.retrieveObject();
            remove(pobj.getBeanInstance());
        }
        catch(Exception e) {
            throw new RemoveException(e.getMessage());
        }
    }

    public void remove(EnterpriseBean bean) throws RemoteException, RemoveException {
        ejbRemove((SessionBean) bean);
    }

    //  StatelessSessionBean, Ƿ true
    public boolean isIdentical(EJBObjectId thisObjectId, EJBObjectId thatObjectId) throws RemoteException {
        return true;
    }

    public EJBPoolableObject retrieveBean(Object key) throws Exception {
        StatelessPoolableObject pobj = (StatelessPoolableObject) instancePool.retrieveObject();
        if(!pobj.isEjbCreated()) { // ĶҪ setSessionContext, ejbCreate
            SessionBean bean = (SessionBean) pobj.getBeanInstance();
            setContext(bean, ejbConext);
            ejbCreate(bean, "create", null);
            pobj.ejbCreated();
        }
        return pobj;
    }

    public void restoreBean(Object key, EJBPoolableObject pobj) {
        instancePool.restoreObject(pobj);
    }

    public EJBObjectId createBean(EJBInvocation invocation) throws Exception {
        if(beanObjectId == null) { // һε create  beanObjectId,ejbContext
            beanObjectId = nextObjectId();
            ejbConext = new SessionContextImpl(beanObjectId, bucketMeta);
        }
        StatelessPoolableObject pobj = (StatelessPoolableObject) instancePool.createObject();
        if(!pobj.isEjbCreated()) { // ĶҪ setSessionContext, ejbCreate
            SessionBean bean = (SessionBean) pobj.getBeanInstance();
            setContext(bean, ejbConext);
            // stateless session bean ֻһûв ejbCreate 
            ejbCreate(bean, "create", null);
            pobj.ejbCreated();
        }
        this.restoreBean(invocation.getObjectId(), pobj);
        return beanObjectId;
    }

	public void syncBean(EJBInvocation invocation) {
		//don't need sync in statless bean
/*
		ClassLoader oldCL = Thread.currentThread().getContextClassLoader();
		ClassLoader cl = this.getEJBClassLoader();
		try {
			Thread.currentThread().setContextClassLoader(cl);
			//invocation.getTargetObject ҪõEJBClassLoader
			EnterpriseBean bean = (EnterpriseBean) invocation.getTargetObject();
			EJBPoolableObject pobj = (EJBPoolableObject) instancePool.createObject();
			pobj.setBeanInstance(bean);
			this.restoreBean(invocation.getObjectId(), pobj);
		}
		catch(Exception e) {
			logger.warn("pool.createObject failed.", e);
		}
		finally {
			Thread.currentThread().setContextClassLoader(oldCL);
		}
*/
	}

	protected void doInit() throws Exception {
        super.doInit();
        instancePool = (StatelessObjectPool)context.getRegistry().getComponentInstance(
                                ComponentName.newInstance(StatelessObjectPool.class),
                                new DependencyPack[]{
                                    new DependencyPack("EJBObjectFactory",
                                                      new ObjectDependency(new StatelessObjectFactory(StatelessPoolableObject.class, this)))});
    }

    protected void doDestroy() throws Exception {
        super.doDestroy();
        instancePool.destroy();
    }
}