/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */

package org.jfox.ejb;

import org.jfox.ioc.common.AbstractComponent;
import org.jfox.pool.AbstractObjectPool;
import org.jfox.pool.ObjectFactory;
import org.jfox.pool.ObjectPool;
import org.jfox.pool.PoolableObject;
import org.jfox.pool.SimpleObjectPool;

/**
 * 洢 StatelessSessionBean Ķأóصĺʹһ SimpleObjectPool
 *
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public class StatelessObjectPool extends AbstractComponent implements ObjectPool, EJBObjectPool {
    private AbstractObjectPool pool = null;
    private EJBObjectFactory factory = null;


    public StatelessObjectPool() {

    }

    public void setEJBObjectFactory(EJBObjectFactory factory) {
        if(factory == null) throw new NullPointerException("factory is null.");
        this.factory = factory;
    }

    public synchronized PoolableObject createObject() throws Exception {
        return retrieveObject();
    }

    public synchronized PoolableObject retrieveObject() throws Exception {
        // not need call ejbCreate, bucket will call it after get instance from the pool
        return pool.retrieveObject();
    }

    /**
     * Ŀ࣬򷵻ظ poolȵõ ejbRemoveͷԴ
     *
     * @param obj
     * @return
     */
    public boolean restoreObject(PoolableObject obj) {
        try {
            return pool.restoreObject(obj);
        }
        catch(Exception e) {
            logger.warn("restoreObject " + obj + " failed, exception is: " + e);
            return false;
        }
    }

    /**
     * жС max, 浽У ejbRemoveϼ
     *
     * @param obj
     * @return
     */
    public boolean removeObject(PoolableObject obj) {
        try {
            return pool.restoreObject(obj);
        }
        catch(Exception e) {
            logger.warn("removeObject " + obj + " failed, exception is: " + e);
            return false;
        }
    }

    public void clear() {
        pool.clear();
    }

    public ObjectFactory getObjectFactory() {
        return factory;
    }

    public String getObjectClass() {
        return factory.getObjectClass().getName();
    }

    public int getWorking() {
        return pool.getWorking();
    }

    public int getRest() {
        return pool.getRest();
    }

    protected void doInit() throws Exception {
        if(factory == null) throw new NullPointerException("factory is null.");
        pool = new SimpleObjectPool(factory, 0, 10);
        pool.init();
    }

    protected void doDestroy() throws Exception {
        pool.destroy();
    }

    public int getInitNum() {
        return pool.getInitNum();
    }

    public int getMaxRest() {
        return pool.getMaxRest();
    }

    public static void main(String[] args) {

    }
}