/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */

package org.jfox.ejb.connector;

import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import javax.ejb.EJBObject;

import org.jfox.ejb.EJBHomeMethod;
import org.jfox.ejb.EJBObjectId;
import org.jfox.ejb.EJBObjectMethod;
import org.jfox.ejb.HandleImpl;
import org.jfox.ioc.connector.AbstractConnectorInvoker;
import org.jfox.ioc.connector.ConnectorHelper;
import org.jfox.ioc.connector.ConnectorInvoker;
import org.jfox.ioc.connector.ConnectorRemote;
import org.jfox.ioc.connector.ObjectId;
import org.jfox.ioc.connector.local.LOCALConnectorRemote;
import org.jfox.ioc.util.Methods;
import org.jfox.jndi.JNDIProperties;

/**
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public class EJBConnectorInvoker extends AbstractConnectorInvoker {

    public EJBConnectorInvoker(ObjectId clientId, ConnectorRemote remote) {
        super(clientId, remote);
    }

    protected Object doInvoke(Object proxy, Method method, Object[] args) throws Throwable {
        EJBObjectId objectId = (EJBObjectId) getClientId();
        if(method.getName().equals(Methods.Equals)) {
            Object obj = args[0];
            if(objectId.isHome()) {
                return new Boolean(objectId.equals(((ConnectorInvoker) Proxy.getInvocationHandler(proxy)).getClientId()));
            }
            else if(obj instanceof EJBObject) {
                return invokeBean(proxy, EJBObjectMethod.IsIdentical, args);
            }
            else {
                return Boolean.FALSE;
            }
        }

        if(remote == null) {
            synchronized(this) {
                // CONTAINER_IDENTITY  Ҫõ ejb  ObjectId  Identity һ£ ejb ڴ
                if(objectId.isCurrentVM()) {
                    remote = LOCALConnectorRemote.getInstance();
                }
                else {
                    // lookup ConnectorService from jndi, Ȼʹö̬װ JMSContainer 
                    remote = ConnectorHelper.lookupConnector(JNDIProperties.getProviderURL());
                }
            }
        }


        if(objectId.isHome()) { // һ EJBHome 
            return invokeHome(proxy, method, args);
        }
        else {
            return invokeBean(proxy, method, args);
        }

    }

    protected Object invokeHome(Object proxy, Method method, Object[] args) throws Exception {
        EJBObjectId objectId = (EJBObjectId) getClientId();
        EJBInvocation invocation = new EJBInvocation(objectId, getMethodHash(method), args);

        if(method.equals(EJBHomeMethod.GetEJBMetaData)) {
            return remote.invoke(invocation);
        }
        else if(method.equals(EJBHomeMethod.GetHomeHandle)) {
//      String homeClass = objectId.getHomeInterfName();
//      return new HomeHandleImpl(homeClass);
            return remote.invoke(invocation);
        }
        else if(method.equals(EJBHomeMethod.RemoveByHandle)) {
            return remote.invoke(invocation);
        }
        else if(method.equals(EJBHomeMethod.RemoveByPrimaryKey)) {
            return remote.invoke(invocation);
        }
        else { // create 
            Object obj = remote.invoke(invocation); // obj  EJBObject  ID
            EJBObjectId _objectId = (EJBObjectId) obj;
            ClassLoader ctxLoader = Thread.currentThread().getContextClassLoader();
            // get the EJBObject class name
            String remoteInterfaceName = _objectId.getRemoteInterfaceName();
            Class remoteInterface = ctxLoader.loadClass(remoteInterfaceName);
            // 챾ص EJBObject 
            return Proxy.newProxyInstance(ctxLoader,
                                          new Class[]{remoteInterface},
                                          new EJBConnectorInvoker(_objectId, remote));
        }
    }

    protected Object invokeBean(Object proxy, Method method, Object[] args) throws Exception {
        EJBObjectId objectId = (EJBObjectId) getClientId();
        EJBInvocation invocation = new EJBInvocation(objectId, getMethodHash(method), args);
        if(method.equals(EJBObjectMethod.GetEJBHome)) {
            return remote.invoke(invocation);
        }
        else if(method.equals(EJBObjectMethod.GetHandle)) {
            ConnectorInvoker invocationHandler = (ConnectorInvoker) Proxy.getInvocationHandler(proxy);
            return new HandleImpl(invocationHandler);
        }
        else if(method.equals(EJBObjectMethod.GetPrimaryKey)) {
            return remote.invoke(invocation);
        }
        else if(method.equals(EJBObjectMethod.IsIdentical)) {
            // Ѳ args[0] תΪEJBObject ġObjectIdֻáObjectId Ƚ
            EJBObject ejbObject = (EJBObject) args[0];
            Object _objectId = ((ConnectorInvoker) (Proxy.getInvocationHandler(ejbObject))).getClientId();
            invocation = new EJBInvocation(objectId, getMethodHash(method), new Object[]{_objectId});
            return remote.invoke(invocation);
        }
        else if(method.equals(EJBObjectMethod.Remove)) {
            return remote.invoke(invocation);
        }
        else {
            return remote.invoke(invocation);
        }
    }


    public static void main(String[] args) {

    }
}
