/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */

package org.jfox.ejb.meta;

import javax.ejb.EJBException;

import org.jfox.ioc.util.XMLUtils;
import org.w3c.dom.Node;

/**
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public class MDBDescriptor extends EJBDescriptor {
    private String destination = null;

    private String acknowledgeMode = null;
    private String destinationType = "javax.jms.Queue";
    private String subscriptionDurability = "Durable";

    public void processXML(Node node) throws EJBDescriptionException {
        setAcknowledgeMode(XMLUtils.getChildNodeValueOf(node, "acknowledge-mode"));

        Node mdd = XMLUtils.getChildNodeOf(node,"message-driven-destination");
        setDestinationType(XMLUtils.getChildNodeValueOf(mdd,"destination-type"));
        setSubscriptionDurability(XMLUtils.getAtrributeValueOf(mdd,"subscription-durability"));

        super.processXML(node);
        String ejbName = getDisplayName();
        int index = ejbName.indexOf(":");
        if(index < 0) {
            throw new EJBDescriptionException("no destination jndi name specified, must use display-name like \"ExampleMDB:destination=queue/Q1\" to specify destination jndi name.");
        }
        String _destination = ejbName.substring(index+1).trim();
        int _index = _destination.indexOf("=");
        if(_index < 0){
            throw new EJBDescriptionException("destination jndi name format error, must be \"ExampleMDB:destination=queue/Q1\" to specify destination jndi name.");
        }
        destination = _destination.substring(_index+1).trim();
    }

    public String getDestination() {
        return destination;
    }

    public void setDestination(String destination) {
        this.destination = destination;
    }

    public String getAcknowledgeMode() {
        return acknowledgeMode;
    }

    public void setAcknowledgeMode(String acknowledgeMode) {
        this.acknowledgeMode = acknowledgeMode;
    }

    public String getDestinationType() {
        return destinationType;
    }

    public void setDestinationType(String destinationType) {
        this.destinationType = destinationType;
    }

    public String getSubscriptionDurability() {
        return subscriptionDurability;
    }

    public void setSubscriptionDurability(String subscriptionDurability) {
        this.subscriptionDurability = subscriptionDurability;
    }

    public String getHomeClassName() {
        throw new EJBException("MessageDriven Bean has no Home Class");
    }

    public String getRemoteClassName() {
        throw new EJBException("MessageDriven Bean has no Remote Class");
    }

    public void setHomeClassName(String s) {
        throw new EJBException("MessageDriven Bean has no Home Class");
    }

    public void setRemoteClassName(String s) {
        throw new EJBException("MessageDriven Bean has no Remote Class");
    }

    public String getJndiName() {
        throw new EJBException("MessageDriven Bean has no JNDI Name");
    }

    public void setJndiName(String s) {
        throw new EJBException("MessageDriven Bean has no JNDI Name");
    }

    public static void main(String[] args) {

    }
}
