/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */
package org.jfox.ejb.meta;

import org.jfox.ioc.util.XMLUtils;
import org.w3c.dom.Node;

/**
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public class SessionDescriptor extends EJBDescriptor {
    public static final String STATEFUL_SESSION = "Stateful";
    public static final String STATELESS_SESSION = "Stateless";
    private String sessionType;

    public SessionDescriptor() {
    }

    public void processXML(Node node) throws EJBDescriptionException {
        setHomeClassName(XMLUtils.getChildNodeValueOf(node, "home"));
        setRemoteClassName(XMLUtils.getChildNodeValueOf(node, "remote"));
        setSessionType(XMLUtils.getChildNodeValueOf(node, "session-type"));
        super.processXML(node);
    }

    public String getSessionType() {
        return sessionType;
    }

    public String getType() {
        return "Session";
    }

    public boolean isStateless() {
        return STATELESS_SESSION.equals(sessionType);
    }

    public void setSessionType(String type) {
        if(!STATEFUL_SESSION.equals(type) && !STATELESS_SESSION.equals(type)) {
            throw new IllegalArgumentException("session-type must be one of \"Stateful\" and \"Stateless\"");
        }
        sessionType = type;
    }


    public static void main(String[] args) {

    }
}

