package org.jfox.ejb.timer;

import java.io.Serializable;
import java.util.TimerTask;
import javax.ejb.EJBException;
import javax.ejb.TimedObject;
import javax.ejb.Timer;

import org.jfox.ejb.Bucket;
import org.jfox.ejb.EJBObjectId;
import org.jfox.ejb.connector.EJBContainerImpl;
import org.jfox.ejb.connector.EJBInvocation;
import org.jfox.ioc.ComponentName;
import org.jfox.ioc.Registry;
import org.jfox.ioc.connector.ConnectorRemote;
import org.jfox.ioc.connector.Container;
import org.jfox.ioc.connector.local.LOCALConnectorRemote;
import org.jfox.ioc.util.Methods;

/**
 * @author <a href="mailto:yy.young@gmail.com">Young Yang</a>
 */

public class EJBTimerTask extends TimerTask {

    private EJBTimerService timerService;
    private EJBTimer timer;


    public EJBTimerTask(EJBTimerService timerService, Serializable info) {
        this.timerService = timerService;
        timer = new EJBTimer(this, info);
    }

    public Timer getTimer() {
        return timer;
    }

    public void run() {
        String ejbName = timerService.getEjbName();

        try {
            Bucket bucket = ((EJBContainerImpl) Registry.getInstance().getComponentInstance(ComponentName.parseString(Container.class.getName() + "@EJB"))).getBucket(ejbName);

            if(!TimedObject.class.isAssignableFrom(bucket.getBeanClass())) {
                throw new EJBException("ejb " + ejbName + " not implements " + TimedObject.class.getName());
            }
            // bucause Timer used by StatelessBean, so objectId can be random
            EJBObjectId objectId = bucket.nextObjectId();
            EJBInvocation invocation = new EJBInvocation(objectId, "" + Methods.getMethodHash(TimedObjectMethod.TimeOut), new Object[]{timer});

            // use LOCALContainerService invoke the timout
            //TODO: Ƿ滻ȷ
//            ContainerRemote local = (ContainerRemote) (Registry.getInstance().getFirstComponentMetabyImplemetation(LOCALContainerService.class).getComponentInstance());
            ConnectorRemote local = LOCALConnectorRemote.getInstance();
            local.invoke(invocation);
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }

    public boolean cancel() {
        timerService.cancel(this);
        return super.cancel();
    }

    EJBTimerService getTimerService() {
        return timerService;
    }

    public static void main(String[] args) {

    }
}

