/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */

package org.jfox.jetty;

import java.io.File;

import org.jfox.ioc.ComponentContext;
import org.jfox.ioc.annotation.Managable;
import org.jfox.ioc.ext.ActiveComponent;
import org.jfox.ioc.ext.ManagableComponent;
import org.jfox.ioc.ext.ServicableComponent;
import org.mortbay.http.HttpContext;
import org.mortbay.j2ee.session.Manager;

/**
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public class JettyWebContainer implements ManagableComponent, ServicableComponent, ActiveComponent, Runnable {
    private JettyServer server;
    private String jettyConfig = "./conf/jetty.xml";

    private boolean started = false;

    public JettyWebContainer() {

    }

    /**
     * because jetty use Thread context classloader,
     * so must start a new thread, and set the thread context class loader to Module LocalClassLoader
     *
     * @throws Exception
     */
    @Managable
    public void start() throws Exception {
        Thread thread = new Thread(this);
        thread.start();

        started = true;
        thread.join();
    }

    @Managable
    public void stop() throws Exception {
        server.stop();
        started = false;
    }


    public void run() {
        // must set the thread context classloader to Module's LocalClassLoader
        Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        try {
            File conf = new File(System.getProperty("jetty.home") + "/" + jettyConfig);
            server = new JettyServer(conf.toURL());
            if(server.getRootWebApp() == "null" || server.getRootWebApp().equals("")) {
                server.setRootWebApp("root");
            }
            server.start();

        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }

    @Managable
    public boolean isStarted() {
        return started && server.isStarted();
    }

    @Managable
    public String getConfiguration() {
        return server == null ? null : server.getConfiguration();
    }

    public String getRootWebApp() {
        return server == null ? null : server.getRootWebApp();
    }

    public void addContext(HttpContext context){
        server.addContext(context);
    }


    public HttpContext getContext(String context){
        return server.getContext(context);
    }

    public boolean removeContext(HttpContext context){
        return server.removeContext(context);
    }

    public void setComponentContext(ComponentContext ctx) {
        System.setProperty("jetty.home", ctx.getModuleDir());

    }
    public Manager getDistributableSessionManagerPrototype() {
        return server.getDistributableSessionManagerPrototype();
    }

    public boolean getForceDistributable() {
        return server.getForceDistributable();
    }
    public static void main(String[] args) throws Exception {
        System.setProperty("jetty.home",".");
        JettyWebContainer jetty = new JettyWebContainer();
        jetty.start();
    }
}


