/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */

package org.jfox.tm;

import java.util.Timer;
import java.util.TimerTask;
import javax.transaction.HeuristicMixedException;
import javax.transaction.HeuristicRollbackException;
import javax.transaction.InvalidTransactionException;
import javax.transaction.NotSupportedException;
import javax.transaction.RollbackException;
import javax.transaction.Status;
import javax.transaction.Synchronization;
import javax.transaction.SystemException;
import javax.transaction.Transaction;
import javax.transaction.xa.XAResource;

/**
 * װһһЩ
 *
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

class TxWrapper implements Transaction {
    /**
     * ????????????????????? timout ?
     */
    private static Timer timer = new Timer(true);
    private static int timeout = 10 * 24 * 60 * 60 * 1000; // 10 days

    private TransactionImpl transaction = null;

    TxWrapper() {

    }

    /**
     * create a transaction & start a timeout
     * ??????????????????????
     */
    void begin() throws SystemException, NotSupportedException {
        transaction = new TransactionImpl();
        timer.schedule((TimerTask) transaction, timeout);
    }

    void setTimeout(int millisecond) {
        timeout = millisecond;
    }


    /**
     * ??? transaction.commit ??? ?????? timeout ???,??????
     *
     * @throws RollbackException
     * @throws HeuristicMixedException
     * @throws HeuristicRollbackException
     * @throws SecurityException
     * @throws SystemException
     */
    public void commit() throws RollbackException, HeuristicMixedException, HeuristicRollbackException, SecurityException, SystemException {
        if(transaction == null) {
            throw new IllegalStateException("no transaction associated with current thread.");
        }
        try {
            transaction.commit();
        }
        finally { // ??????????????????????
            ((TimerTask) transaction).cancel();
        }
    }

    public boolean delistResource(XAResource xaResource, int flag) throws IllegalStateException, SystemException {
        if(transaction == null) {
            throw new IllegalStateException("no transaction associated with current thread.");
        }
        return transaction.delistResource(xaResource, flag);
    }

    public boolean enlistResource(XAResource xaResource) throws RollbackException, IllegalStateException, SystemException {
        if(transaction == null) {
            throw new IllegalStateException("no transaction associated with current thread.");
        }
        return transaction.enlistResource(xaResource);
    }

    public int getStatus() throws SystemException {
        if(transaction == null) {
            return Status.STATUS_NO_TRANSACTION;
        }
        else {
            return transaction.getStatus();
        }
    }

    public void registerSynchronization(Synchronization synchronization) throws RollbackException, IllegalStateException, SystemException {
        if(transaction == null) {
            throw new IllegalStateException("no transaction associated with current thread.");
        }
        if(synchronization == null) return;
        transaction.registerSynchronization(synchronization);
    }

    /**
     * ??? transaction.rollback ??????????? timeout ???????????????
     *
     * @throws IllegalStateException
     * @throws SystemException
     */
    public void rollback() throws IllegalStateException, SystemException {
        if(transaction == null) {
            throw new IllegalStateException("no transaction associated with current thread.");
        }
        try {
            transaction.rollback();
        }
        finally {
            ((TimerTask) transaction).cancel();
        }
    }

    public void setRollbackOnly() throws IllegalStateException, SystemException {
        if(transaction == null) {
            throw new IllegalStateException("no transaction associated with current thread.");
        }
        transaction.setRollbackOnly();
    }

    public String toString() {
        if(transaction == null) {
            return "No Transaction.";
        }
        else {
            return transaction.toString();
        }
    }

    Transaction suspend() throws SystemException {
        if(transaction == null) {
            throw new IllegalStateException("no transaction associated with current thread.");
        }
        Transaction suspendedTransaction = transaction;
        transaction.suspend();
        transaction = null;
        return suspendedTransaction;
    }

    void resume(Transaction trans) throws InvalidTransactionException, IllegalStateException, SystemException {
        if(trans == null || !(trans instanceof TransactionImpl)) {
            throw new InvalidTransactionException(trans + " is not a instance of " + TransactionImpl.class.getName());
        }
        transaction = (TransactionImpl) trans;
        try {
            transaction.resume();
        }
        catch(RollbackException e) {
            throw new IllegalStateException("transaction already rolled back, " + e.getMessage());
        }
    }
}
